@@ -1,7 +1,7 @@
 # Copyright (c) 2003-2004 Hyriand. All rights reserved.
 #
 import gtk
-import os
+import os, sys
 from pynicotine.utils import version
 import imagedata
 
@@ -26,6 +26,11 @@ class AboutDialog(GenericAboutDialog):
 		img.set_from_pixbuf(pixbuf)
 		label = gtk.Label(_("""Nicotine+ %s
 
+Copyright (c) 2007 daelstorm
+http://nicotine-plus.sourceforge.net/
+daelstorm@gmail.com
+
+Based on code from Nicotine
 Copyright (c) 2003 Hyriand
 http://nicotine.thegraveyard.org/
 hyriand@thegraveyard.org
@@ -33,20 +38,89 @@ hyriand@thegraveyard.org
 Based on code from the PySoulSeek project
 Copyright (c) 2001-2003 Alexander Kanavin
 
-With additions by daelstorm and others (c) 2006
-http://nicotine-plus.sourceforge.net/
-
 Released under the GNU Public License
 
 See MAINTAINERS file for the list of contributors""") % version)
-		label.set_justify(gtk.JUSTIFY_CENTER)
+		label.set_justify(gtk.JUSTIFY_LEFT)
 		vbox = gtk.VBox()
 		vbox.pack_start(img, False, False)
 		hbox = gtk.HBox()
 		hbox.set_spacing(10)
 		hbox.pack_start(vbox, True, True)
 		hbox.pack_start(label, True, True)
+		sys.version_info
+		pythonversion = "%d.%d.%d" % (sys.version_info[0], sys.version_info[1], sys.version_info[2])
+		
+		self.frame1 = gtk.Frame()
+		self.frame1.show()
+		self.frame1.set_shadow_type(gtk.SHADOW_ETCHED_IN)
+		
+		
+		self.vbox2 = gtk.VBox()
+		self.vbox2.set_spacing(5)
+		self.vbox2.set_border_width(5)
+		self.frame1.add(self.vbox2)
+		
+		self.label17 = gtk.Label(_("Dependencies"))
+		self.label17.set_padding(0, 0)
+		self.label17.set_line_wrap(False)
+		self.label17.show()
+		self.frame1.set_label_widget(self.label17)
+		
+		hboxpython = gtk.HBox(5)
+		hboxpython.show()
+		python = gtk.Label("Python:")
+		python.set_alignment(0, 0.5)
+		python.show()
+		
+		VersionPython = gtk.Label(pythonversion)
+		VersionPython.set_alignment(0, 0.5)
+		VersionPython.show()
+		
+		hboxpython.pack_start(python, True, True)
+		hboxpython.pack_start(VersionPython, True, True)
+		
+		hboxgtk = gtk.HBox(5)
+		hboxgtk.show()
+		
+		gtkversion = "%d.%d.%d" % (gtk.gtk_version[0], gtk.gtk_version[1], gtk.gtk_version[2])
+		VersionGTK = gtk.Label(gtkversion)
+		
+		gtkplus = gtk.Label("GTK+:")
+		gtkplus.set_alignment(0, 0.5)
+		gtkplus.show()
+		
+		VersionGTK.set_alignment(0, 0.5)
+		VersionGTK.show()
+		
+		hboxgtk.pack_start(gtkplus, True, True)
+		hboxgtk.pack_start(VersionGTK, True, True)
+		
+		hboxpygtk = gtk.HBox(5)
+		hboxpygtk.show()
+		
+		pygtkversion = "%d.%d.%d" % (gtk.pygtk_version[0], gtk.pygtk_version[1], gtk.pygtk_version[2])
+		VersionPyGTK = gtk.Label(pygtkversion)
+			
+		pygtkplus = gtk.Label("PyGTK+:")
+		pygtkplus.set_alignment(0, 0.5)
+		pygtkplus.show()
+		
+		VersionPyGTK.set_alignment(0, 0.5)
+		VersionPyGTK.show()
+		
+		hboxpygtk.pack_start(pygtkplus, True, True)
+		hboxpygtk.pack_start(VersionPyGTK, True, True)
+		
+		self.vbox2.pack_start(hboxpython, True, True)
+		self.vbox2.pack_start(hboxgtk, True, True)
+		self.vbox2.pack_start(hboxpygtk, True, True)
+		
 		self.vbox.pack_start(hbox, True, True)
+		self.vbox.pack_start(self.frame1, True, True)
+		
+		
+		
 		self.show_all()
 
 class AboutFiltersDialog(GenericAboutDialog):
@@ -79,7 +153,7 @@ free.
 To set the filter, press Enter. This will apply to any existing results, and any
 more that are returned. To filter in a different way, just set the relevant terms.
 You do not need to do another search to apply a different filter."""))
-		label.set_justify(gtk.JUSTIFY_CENTER)
+		label.set_justify(gtk.JUSTIFY_LEFT)
 		self.vbox.pack_start(label)
 		self.show_all()
 
@@ -166,6 +240,8 @@ class AboutPrivateDialog(GenericTableDialog):
 	        "/bsearch /bs '" + _("query")+"'", _("Search the buddy list for 'query'"),
 	        "/usearch /us '" + _("query")+"'", _("Search a user's shares for 'query'"),
 	        "", "",
+		"/join /j '" + _("room")+"'", _("Join room 'room'"),
+		"", "",
 	        "/rescan", _("Rescan shares"),
 	        "/away /a", _("Toggles your away status"),
 	        "/quit /q", _("Quit Nicotine"),
@@ -173,3 +249,18 @@ class AboutPrivateDialog(GenericTableDialog):
 
 	def __init__(self, parent):
 		GenericTableDialog.__init__(self, parent, _("About private chat commands"))
+
+class AboutDependenciesDialog(GenericTableDialog):
+	items = [
+	        _("Sound Effects"), _("Gstreamer-python: http://gstreamer.freedesktop.org/modules/gst-python.html \nSoX: http://sox.sourceforge.net/ \nAny other command-executable OGG player"),
+		"", "",
+		_("Spell Checking"), _("Libsexy, sexy-python: http://www.chipx86.com/wiki/Libsexy"),
+		_("Speed Up"), _("Psyco: http://psyco.sourceforge.net/"),
+	        _("IP Address Geolocation"), _("GeoIP-Python: http://www.maxmind.com/app/python"),
+		_("OGG Metadata"), _("PyVorbis: http://ekyo.nerim.net/software/pyogg/ (Warning: May be unstable)"),
+	        
+
+	]
+
+	def __init__(self, parent):
+		GenericTableDialog.__init__(self, parent, _("About optional dependencies"))
\ No newline at end of file
