@@ -3,10 +3,10 @@
 import gtk
 import gobject
 import locale
-
+import pango
 from pynicotine import slskmessages
 from nicotine_glade import ChatRoomTab
-from utils import InitialiseColumns, AppendLine, PopupMenu, FastListModel, string_sort_func, WriteLog, int_sort_func, Humanize, expand_alias, EncodingsMenu, SaveEncoding
+from utils import InitialiseColumns, AppendLine, PopupMenu, FastListModel, string_sort_func, WriteLog, int_sort_func, Humanize, expand_alias, EncodingsMenu, SaveEncoding, PressHeader
 from pynicotine.utils import _
 from ticker import Ticker
 
@@ -16,6 +16,7 @@ def GetCompletion(part, list):
 		if match in matches:
 			continue
 		if match[:len(part)] == part and len(match) > len(part):
+			#print match
 			matches.append(match)
 	
 	if len(matches) == 0:
@@ -62,8 +63,8 @@ class RoomsControl:
 		self.autojoin = 1
 		
 		cols = InitialiseColumns(frame.roomlist.RoomsList,
-			[_("Room"), 150, "text"],
-			[_("Users"), -1, "text"],
+			[_("Room"), 150, "text", self.frame.CellDataFunc],
+			[_("Users"), -1, "text", self.frame.CellDataFunc],
 		)
 		
 		for ix in range(len(cols)):
@@ -72,22 +73,55 @@ class RoomsControl:
 		
 		cols[1].set_sort_indicator(True)
 		cols[1].set_sort_order(gtk.SORT_DESCENDING)
-		
+		for i in range (2):
+			parent = cols[i].get_widget().get_ancestor(gtk.Button)
+			if parent:
+				parent.connect('button_press_event', PressHeader)
+
 		self.roomsmodel = RoomsListModel([])
 		frame.roomlist.RoomsList.set_model(self.roomsmodel)
 		
 		self.popup_room = None
 		self.popup_menu = PopupMenu().setup(
-			( _("Join room"), self.OnPopupJoin ),
-			( _("Leave room"), self.OnPopupLeave, ),
+			("#" + _("Join room"), self.OnPopupJoin, gtk.STOCK_JUMP_TO ),
+			("#" + _("Leave room"), self.OnPopupLeave, gtk.STOCK_CLOSE),
 			( "", None ),
-			( _("Refresh"), self.OnPopupRefresh ),
+			("#" + _("Refresh"), self.OnPopupRefresh, gtk.STOCK_REFRESH ),
 		)
 		frame.roomlist.RoomsList.connect("button_press_event", self.OnListClicked)
 		frame.roomlist.RoomsList.set_headers_clickable(True)
 		
 		self.frame.ChatNotebook.connect("switch-page", self.OnSwitchPage)
-
+		try:
+			self.frame.ChatNotebook.connect("page-reordered", self.OnReorderedPage)
+		except:
+			# No PyGTK 2.10! Gosh, you really need to get with the times!
+			pass
+		self.frame.SetTextBG(self.frame.roomlist.RoomsList)
+		self.frame.SetTextBG(self.frame.roomlist.CreateRoomEntry)
+		
+	def OnReorderedPage(self, notebook, page, page_num, force=0):
+		room_tab_order = {}
+		# Find position of opened autojoined rooms
+		for name, room in self.joinedrooms.items():
+			if name not in self.frame.np.config.sections["server"]["autojoin"]:
+				continue
+			room_tab_order [ notebook.page_num(room.Main) ] = name
+		pos = 1000
+		# Add closed autojoined rooms as well
+		for name in self.frame.np.config.sections["server"]["autojoin"]:
+			if not self.joinedrooms.has_key(name):
+				room_tab_order [ pos ] = name
+				pos += 1
+		# Sort by "position"
+		rto = room_tab_order.keys()
+		rto.sort()
+		new_autojoin = []
+		for roomplace in rto:
+			new_autojoin.append(room_tab_order[roomplace])
+		# Save
+		self.frame.np.config.sections["server"]["autojoin"] = new_autojoin
+		
 	def OnSwitchPage(self, notebook, page, page_num, force=0):
 		if self.frame.notebook1.get_current_page() != 0 and not force:
 			return
@@ -96,7 +130,7 @@ class RoomsControl:
 			if room.Main == page:
 				gobject.idle_add(room.entry3.grab_focus)
 				# Remove hilite
-				if name in self.frame.tray_status["hilites"]["rooms"]:
+				if name in self.frame.TrayApp.tray_status["hilites"]["rooms"]:
 					self.frame.ClearNotification("rooms", None, name)
 
 			
@@ -171,6 +205,10 @@ class RoomsControl:
 		self.joinedrooms[msg.room] = tab
 		self.frame.ChatNotebook.append_page(tab.Main, msg.room, tab.OnLeave)
 		
+		self.frame.searchroomslist[msg.room] = self.frame.RoomSearchCombo_List.append([msg.room])
+		tab.CountUsers()
+		
+
 	def SetRoomList(self, msg):
 		if self.autojoin:
 			self.autojoin = 0
@@ -215,8 +253,18 @@ class RoomsControl:
 		self.joinedrooms[msg.room].SayChatRoom(msg, text)
 	
 	def UpdateColours(self):
+		self.frame.SetTextBG(self.frame.roomlist.RoomsList)
+		self.frame.SetTextBG(self.frame.roomlist.CreateRoomEntry)
 		for room in self.joinedrooms.values():
-			room.UpdateColours()
+			room.ChangeColours()
+			
+	def saveColumns(self):
+		for room in self.frame.np.config.sections["columns"]["chatrooms"].keys()[:]:
+			if room not in self.joinedrooms.keys():
+				del self.frame.np.config.sections["columns"]["chatrooms"][room]
+		for room in self.joinedrooms.values():
+			room.saveColumns()
+		
 
 	def LeaveRoom(self, msg):
 		room = self.joinedrooms[msg.room]
@@ -226,12 +274,15 @@ class RoomsControl:
 		self.frame.ChatNotebook.remove_page(room.Main)
 		room.destroy()
 		del self.joinedrooms[msg.room]
+		self.frame.RoomSearchCombo_List.remove(self.frame.searchroomslist[msg.room])
 	
 	def ConnClose(self):
 		self.roomsmodel = None
 		self.frame.roomlist.RoomsList.set_model(None)
+		
 		for room in self.joinedrooms.values():
 			room.ConnClose()
+
 		self.autojoin = 1
 
 def TickDialog(parent, default = ""):
@@ -295,7 +346,7 @@ class ChatRoom(ChatRoomTab):
 		self.lines = []
 		self.logfile = None
 		self.leaving = 0
-
+		config = self.frame.np.config.sections
 		if not self.frame.np.config.sections["ticker"]["hide"]:
 			self.Ticker.show()
 
@@ -304,9 +355,9 @@ class ChatRoom(ChatRoomTab):
 			self.tlux_chat = lambda: self.ChatScroll.get_window(gtk.TEXT_WINDOW_TEXT)
 			self.frame.translux.subscribe(self.RoomLog, self.tlux_roomlog)
 			self.frame.translux.subscribe(self.ChatScroll, self.tlux_chat)
-	        	self.RoomLog.get_parent().get_vadjustment().connect("value-changed", lambda *args: self.RoomLog.queue_draw())
-	        	self.ChatScroll.get_parent().get_vadjustment().connect("value-changed", lambda *args: self.ChatScroll.queue_draw())
-	        	self.ChatScroll.get_parent().get_hadjustment().connect("value-changed", lambda *args: self.ChatScroll.queue_draw())
+			self.RoomLog.get_parent().get_vadjustment().connect("value-changed", lambda *args: self.RoomLog.queue_draw())
+			self.ChatScroll.get_parent().get_vadjustment().connect("value-changed", lambda *args: self.ChatScroll.queue_draw())
+			self.ChatScroll.get_parent().get_hadjustment().connect("value-changed", lambda *args: self.ChatScroll.queue_draw())
 
 		self.Elist = {}
 		self.encoding, m = EncodingsMenu(self.frame.np, "roomencoding", room)
@@ -320,29 +371,46 @@ class ChatRoom(ChatRoomTab):
 			self.Elist[item[1]] = self.EncodingStore.append([item[1], item[0] ])
 			if self.encoding == item[1]:
 				self.Encoding.set_active_iter(self.Elist[self.encoding])
-
+		if self.frame.SEXY and self.frame.np.config.sections["ui"]["spellcheck"]:
+			import sexy
+			self.vbox6.remove(self.entry3)
+			self.entry3.destroy()
+			self.entry3 = sexy.SpellEntry()
+			self.entry3.show()
+        		self.entry3.connect("activate", self.OnEnter)
+        		self.entry3.connect("key_press_event", self.OnKeyPress)
+			self.vbox6.pack_start(self.entry3, False, False, 0)
+	
 		self.Log.set_active(self.frame.np.config.sections["logging"]["chatrooms"])
 		
-		self.UpdateColours()
+		
 		
 		if room in self.frame.np.config.sections["server"]["autojoin"]:
 			self.AutoJoin.set_active(True)
 			
 		cols = InitialiseColumns(self.UserList, 
-			["", -1, "pixbuf"],
-			[_("User"), 100, "text"],
-			[_("Speed"), 0, "text"],
-			[_("Files"), 0, "text"],
+			[_("Status"), 20, "pixbuf"],
+			[_("User"), 100, "text", self.frame.CellDataFunc],
+			[_("Speed"), 0, "text", self.frame.CellDataFunc],
+			[_("Files"), 0, "text", self.frame.CellDataFunc],
 		)
 		cols[0].set_sort_column_id(4)
 		cols[1].set_sort_column_id(1)
 		cols[2].set_sort_column_id(5)
 		cols[3].set_sort_column_id(6)
-		
+		cols[0].get_widget().hide()
+		if not config["columns"]["chatrooms"].has_key(room):
+			config["columns"]["chatrooms"][room] = [1, 1, 1, 1]
+		for i in range (4):
+			parent = cols[i].get_widget().get_ancestor(gtk.Button)
+			if parent:
+				parent.connect('button_press_event', PressHeader)
+			# Read Show / Hide column settings from last session
+			cols[i].set_visible(config["columns"]["chatrooms"][room][i])
 		self.users = {}
 
 		self.usersmodel = gtk.ListStore(gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_INT, gobject.TYPE_INT, gobject.TYPE_INT)
-		self.usersmodel.set_sort_column_id(1, gtk.SORT_ASCENDING)
+		
 		
 		for user in users.keys():
 			img = self.frame.GetStatusImage(users[user].status)
@@ -350,17 +418,22 @@ class ChatRoom(ChatRoomTab):
 			hfiles = Humanize(users[user].files)
 			iter = self.usersmodel.append([img, user, hspeed, hfiles, users[user].status, users[user].avgspeed, users[user].files])
 			self.users[user] = iter
-		
+		self.usersmodel.set_sort_column_id(1, gtk.SORT_ASCENDING)
+		self.UpdateColours()
 		self.UserList.set_model(self.usersmodel)
-	
+		self.UserList.set_property("rules-hint", True)
+		
 		self.popup_menu = popup = PopupMenu(self.frame)
 		popup.setup(
+			("USER", ""),
+			("", None),
 			("#" + _("Send _message"), popup.OnSendMessage, gtk.STOCK_EDIT),
 			("", None),
 			("#" + _("Show IP a_ddress"), popup.OnShowIPaddress, gtk.STOCK_NETWORK),
 			("#" + _("Get user i_nfo"), popup.OnGetUserInfo, gtk.STOCK_DIALOG_INFO),
 			("#" + _("Brow_se files"), popup.OnBrowseUser, gtk.STOCK_HARDDISK),
 			("#" + _("Gi_ve privileges"), popup.OnGivePrivileges, gtk.STOCK_JUMP_TO),
+			("", None),
 			("$" + _("_Add user to list"), popup.OnAddToList),
 			("$" + _("_Ban this user"), popup.OnBanUser),
 			("$" + _("_Ignore this user"), popup.OnIgnoreUser),
@@ -371,11 +444,32 @@ class ChatRoom(ChatRoomTab):
 		self.vbox6.set_focus_child(self.entry3)
 		
 		self.logpopupmenu = PopupMenu(self.frame).setup(
+			("#" + _("Find"), self.OnFindLogWindow, gtk.STOCK_FIND),
+			("", None),
+			("#" + _("Copy"), self.OnCopyRoomLog, gtk.STOCK_COPY),
 			("#" + _("Copy All"), self.OnCopyAllRoomLog, gtk.STOCK_COPY),
+			("", None),
 			("#" + _("Clear log"), self.OnClearRoomLog, gtk.STOCK_CLEAR),
 		)
 		self.RoomLog.connect("button-press-event", self.OnPopupRoomLogMenu)
 		
+		self.chatpopmenu = PopupMenu(self.frame).setup(
+			("#" + _("Find"), self.OnFindChatLog, gtk.STOCK_FIND),
+			("", None),
+			("#" + _("Copy"), self.OnCopyChatLog, gtk.STOCK_COPY),
+			("#" + _("Copy All"), self.OnCopyAllChatLog, gtk.STOCK_COPY),
+			("", None),
+			("#" + _("Clear log"), self.OnClearChatLog, gtk.STOCK_CLEAR),
+		)
+		self.ChatScroll.connect("button-press-event", self.OnPopupChatRoomMenu)
+		
+	def OnFindLogWindow(self, widget):
+
+		self.frame.OnFindTextview(widget, self.RoomLog)
+		
+	def OnFindChatLog(self, widget):
+		self.frame.OnFindTextview(widget, self.ChatScroll)
+		
 	def get_custom_widget(self, id, string1, string2, int1, int2):
 		if id == "Ticker":
 			t = Ticker()
@@ -405,9 +499,9 @@ class ChatRoom(ChatRoomTab):
 			return
 		
 		self.popup_menu.set_user(user)
-		items[6].set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
-		items[7].set_active(user in self.frame.np.config.sections["server"]["banlist"])
-		items[8].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
+		items[9].set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
+		items[10].set_active(user in self.frame.np.config.sections["server"]["banlist"])
+		items[11].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
 		self.popup_menu.popup(None, None, None, event.button, event.time)
 		
 	def TickerSet(self, msg):
@@ -424,26 +518,22 @@ class ChatRoom(ChatRoomTab):
 		
 	def SayChatRoom(self, msg, text):
 		login = self.frame.np.config.sections["server"]["login"]
-		if msg.user == login:
+		user = msg.user
+		if user == login:
 			tag = self.tag_local
 		elif text.upper().find(login.upper()) > -1:
 			tag = self.tag_hilite
-		#elif msg.user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]]:
-			#print msg.user, "is a buddy"
-		#elif msg.user in self.frame.np.config.sections["server"]["banlist"]:
-			#print msg.user, "is a banned"
-		#elif msg.user in self.frame.np.config.sections["server"]["ignorelist"]:
-			#print msg.user, "is ignored"
+
 		else:
 			tag = self.tag_remote
 		
-		if msg.user != login and tag == self.tag_hilite:
+		if user != login and tag == self.tag_hilite:
 			self.frame.ChatNotebook.request_hilite(self.Main)
 			self.frame.ChatRequestIcon(1)
 			# add hilite to trayicon
 			if self.frame.ChatNotebook.get_current_page() != self.frame.ChatNotebook.page_num(self.roomsctrl.joinedrooms[self.room].Main) or self.frame.notebook1.get_current_page() != 0:
-				if self.room not in self.frame.tray_status["hilites"]["rooms"]:
-					self.frame.Notification("rooms", msg.user, self.room)
+				if self.room not in self.frame.TrayApp.tray_status["hilites"]["rooms"]:
+					self.frame.Notification("rooms", user, self.room)
 			#else:
 				#self.MainWindow.set_urgency_hint(False)
 				
@@ -453,10 +543,10 @@ class ChatRoom(ChatRoomTab):
 			self.frame.ChatRequestIcon(0)
 			
 		if text[:4] == "/me ":
-			line = "* %s %s" % (msg.user, text[4:])
+			line = "* %s %s" % (user, text[4:])
 			tag = self.tag_me
 		else:
-			line = "[%s] %s" % (msg.user, text)
+			line = "[%s] %s" % (user, text)
 		
 		if len(self.lines) >= 400:
 			buffer = self.ChatScroll.get_buffer()
@@ -466,8 +556,13 @@ class ChatRoom(ChatRoomTab):
 			del self.lines[0:200]
 
 		line = "\n-- ".join(line.split("\n"))
-
-		self.lines.append(AppendLine(self.ChatScroll, self.frame.np.decode(line, self.encoding), tag))
+		
+		color = self.getUserStatusColor(self.usersmodel.get_value(self.users[user], 4))
+		if self.tag_users.has_key(user):
+			self.changecolour(self.tag_users[user], color)
+		else:
+			self.tag_users[user] = self.makecolour(self.ChatScroll.get_buffer(), color, user)
+		self.lines.append(AppendLine(self.ChatScroll, self.frame.np.decode(line, self.encoding), tag, username=user, usertag=self.tag_users[user]))
 		if self.Log.get_active():
 			self.logfile = WriteLog(self.logfile, self.frame.np.config.sections["logging"]["logsdir"], self.room, line)
 			
@@ -607,14 +702,36 @@ class ChatRoom(ChatRoomTab):
 		hfiles = Humanize(userdata.files)
 		iter = self.usersmodel.append([img, username, hspeed, hfiles, userdata.status, userdata.avgspeed, userdata.files])
 		self.users[username] = iter
-	
+		color = self.getUserStatusColor(userdata.status)
+		if username in self.tag_users.keys():
+			
+			self.changecolour(self.tag_users[username], color)
+		else:
+			self.tag_users[username] = self.makecolour(self.ChatScroll.get_buffer(), color, username=username)
+		self.CountUsers()
+		
 	def UserLeftRoom(self, username):
 		if not self.users.has_key(username):
 			return
 		AppendLine(self.RoomLog, _("%s left the room") % username, self.tag_log)
 		self.usersmodel.remove(self.users[username])
 		del self.users[username]
-	
+		if username in self.tag_users.keys():
+			color = self.getUserStatusColor(-1)
+			self.changecolour(self.tag_users[username], color)
+		self.CountUsers()
+		
+	def CountUsers(self):
+		numusers = len(self.users.keys())
+		if numusers > 1:
+			self.LabelPeople.show()
+			self.LabelPeople.set_text(_("%i people in room") % numusers)
+		elif numusers == 1:
+			self.LabelPeople.show()
+			self.LabelPeople.set_text(_("You are alone"))
+		else:
+			self.LabelPeople.hide()
+		
 	def GetUserStats(self, user, avgspeed, files):
 		if not self.users.has_key(user):
 			return
@@ -631,37 +748,167 @@ class ChatRoom(ChatRoomTab):
 		else:
 			action = _("%s has returned")
 		AppendLine(self.RoomLog, action % user, self.tag_log)
+		if user in self.tag_users.keys():
+			color = self.getUserStatusColor(status)
+			self.changecolour(self.tag_users[user], color)
 		self.usersmodel.set(self.users[user], 0, img, 4, status)
-
-	def UpdateColours(self):
-		def makecolour(buffer, colour):
-			colour = self.frame.np.config.sections["ui"][colour]
-			font = self.frame.np.config.sections["ui"]["chatfont"]
-			if colour:
-				return buffer.create_tag(foreground = colour, font=font)
+		
+	def makecolour(self, buffer, colour, username=None):
+		colour = self.frame.np.config.sections["ui"][colour]
+		font =  self.frame.np.config.sections["ui"]["chatfont"]
+		
+		if colour:
+			tag = buffer.create_tag(foreground = colour, font=font)
+		else:
+			tag = buffer.create_tag( font=font)
+		if username is not None:
+			usernamestyle = self.frame.np.config.sections["ui"]["usernamestyle"]
+			
+			#tag.set_property("weight",  pango.WEIGHT_BOLD)
+			if usernamestyle == "bold":
+				tag.set_property("weight",  pango.WEIGHT_BOLD)
 			else:
-				return buffer.create_tag( font=font)
+				tag.set_property("weight",  pango.WEIGHT_NORMAL)
+			if usernamestyle == "italic":
+				tag.set_property("style",  pango.STYLE_ITALIC)
+			else:
+				tag.set_property("style",  pango.STYLE_NORMAL)
+			if usernamestyle == "underline":
+				tag.set_property("underline", pango.UNDERLINE_SINGLE)
+			else:
+				tag.set_property("underline", pango.UNDERLINE_NONE)
 				
+			tag.connect("event", self.UserNameEvent, username)
+			tag.last_event_type = -1
+		return tag
+		
+	def UserNameEvent(self, tag, widget, event, iter, user):
+
+		if tag.last_event_type == gtk.gdk.BUTTON_PRESS and event.type == gtk.gdk.BUTTON_RELEASE and event.button in (1, 2):
+			self.popup_menu.set_user(user)
+			items = self.popup_menu.get_children()
+			# Chat, Userlists use the normal popup system
+			items[9].set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
+			items[10].set_active(user in self.frame.np.config.sections["server"]["banlist"])
+			items[11].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
+			self.popup_menu.popup(None, None, None, event.button, event.time)
+		tag.last_event_type = event.type
+		
+	def UpdateColours(self):
+
+		map = self.ChatScroll.get_style().copy()
+		self.backupcolor = map.text[gtk.STATE_NORMAL]
 		buffer = self.ChatScroll.get_buffer()
-		self.tag_remote = makecolour(buffer, "chatremote")
-		self.tag_local = makecolour(buffer, "chatlocal")
-		self.tag_me = makecolour(buffer, "chatme")
-		self.tag_hilite = makecolour(buffer, "chathilite")
-		buffer = self.RoomLog.get_buffer()
-		self.tag_log = makecolour(buffer, "chatremote")
-	
+		self.tag_remote = self.makecolour(buffer, "chatremote")
+		self.tag_local = self.makecolour(buffer, "chatlocal")
+		self.tag_me = self.makecolour(buffer, "chatme")
+		self.tag_hilite = self.makecolour(buffer, "chathilite")
+		self.tag_users = {}
+		for user in self.users:
+			status = self.usersmodel.get_value(self.users[user], 4)
+			color = self.getUserStatusColor(status)
+			if user in self.tag_users.keys():
+				self.changecolour(self.tag_users[username], color)
+			else:
+				self.tag_users[user] = self.makecolour(buffer, color, user)
+		logbuffer = self.RoomLog.get_buffer()
+		self.tag_log = self.makecolour(logbuffer, "chatremote")
+		
+		self.frame.SetTextBG(self.ChatScroll)
+		self.frame.SetTextBG(self.RoomLog)
+		self.frame.SetTextBG(self.UserList)
+		
+		self.frame.SetTextBG(self.entry3)
+		
+		
+	def getUserStatusColor(self, status):
+		if status == 1:
+			color = "useraway"
+		elif status == 2:
+			color = "useronline"
+		else:
+			color = "useroffline"
+		return color
+		
+	def changecolour(self, tag, colour):
+		if self.frame.np.config.sections["ui"].has_key(colour):
+			color = self.frame.np.config.sections["ui"][colour]
+		else:
+			color = ""
+		font = self.frame.np.config.sections["ui"]["chatfont"]
+		
+		if color == "":
+			color = self.backupcolor
+		else:
+			color = gtk.gdk.color_parse(color)
+		tag.set_property("foreground-gdk", color)
+		tag.set_property("font", font)
+		# Hotspots
+		if colour in ["useraway", "useronline", "useroffline"]:
+			usernamestyle = self.frame.np.config.sections["ui"]["usernamestyle"]
+			if usernamestyle == "bold":
+				tag.set_property("weight",  pango.WEIGHT_BOLD)
+			else:
+				tag.set_property("weight",  pango.WEIGHT_NORMAL)
+			if usernamestyle == "italic":
+				tag.set_property("style",  pango.STYLE_ITALIC)
+			else:
+				tag.set_property("style",  pango.STYLE_NORMAL)
+			if usernamestyle == "underline":
+				tag.set_property("underline", pango.UNDERLINE_SINGLE)
+			else:
+				tag.set_property("underline", pango.UNDERLINE_NONE)
+		
+			
+	def ChangeColours(self):
+		
+		map = self.ChatScroll.get_style().copy()
+		self.backupcolor = map.text[gtk.STATE_NORMAL]
+			
+		self.changecolour(self.tag_remote, "chatremote")
+		self.changecolour(self.tag_local, "chatlocal")
+		self.changecolour(self.tag_me, "chatme")
+		self.changecolour(self.tag_hilite, "chathilite")
+		self.changecolour(self.tag_log, "chatremote")
+
+		for user in self.users.keys():
+			color = self.getUserStatusColor(self.usersmodel.get_value(self.users[user], 4))
+			if user in self.tag_users.keys():
+				self.changecolour(self.tag_users[user], color)
+			else:
+				self.tag_users[user] = self.makecolour(buffer, color, user)
+			
+		self.frame.SetTextBG(self.ChatScroll)
+		self.frame.SetTextBG(self.RoomLog)
+		self.frame.SetTextBG(self.UserList)
+		self.frame.SetTextBG(self.entry3)
+				
 	def OnLeave(self, widget = None):
 		if self.leaving:
 			return
 		self.frame.np.queue.put(slskmessages.LeaveRoom(self.room))
 		self.Leave.set_sensitive(False)
 		self.leaving = 1
-
+		config = self.frame.np.config.sections
+		if config["columns"]["chatrooms"].has_key(self.room):
+			del config["columns"]["chatrooms"][self.room]
+			
+	def saveColumns(self):
+		columns = []
+		for column in self.UserList.get_columns():
+			columns.append(column.get_visible())
+		self.frame.np.config.sections["columns"]["chatrooms"][self.room] = columns
+		
+		
 	def ConnClose(self):
 		AppendLine(self.ChatScroll, _("--- disconnected ---"), self.tag_hilite)
 		self.usersmodel.clear()
 		self.users = {}
-	
+		self.CountUsers()
+		config = self.frame.np.config.sections
+  		if not self.AutoJoin.get_active() and config["columns"]["chatrooms"].has_key(self.room):
+			del config["columns"]["chatrooms"][self.room]
+		
 	def Rejoined(self, users):
 		for user in users.keys():
 			if self.users.has_key(user):
@@ -672,6 +919,7 @@ class ChatRoom(ChatRoomTab):
 			iter = self.usersmodel.append([img, user, hspeed, hfiles, users[user].status, users[user].avgspeed, users[user].files])
 			self.users[user] = iter
 		AppendLine(self.ChatScroll, _("--- reconnected ---"), self.tag_hilite)
+		self.CountUsers()
 
 	def OnAutojoin(self, widget):
 		autojoin = self.frame.np.config.sections["server"]["autojoin"]
@@ -730,7 +978,14 @@ class ChatRoom(ChatRoomTab):
 		if encoding != self.encoding:
 			self.encoding = encoding
 			SaveEncoding(self.frame.np, "roomencoding", self.room, self.encoding)
-
+	
+	def OnPopupChatRoomMenu(self, widget, event):
+		if event.button != 3:
+			return False
+		widget.emit_stop_by_name("button-press-event")
+		self.chatpopmenu.popup(None, None, None, event.button, event.time)
+		return True
+	
 	def OnPopupRoomLogMenu(self, widget, event):
 		if event.button != 3:
 			return False
@@ -743,9 +998,31 @@ class ChatRoom(ChatRoomTab):
 		log = self.RoomLog.get_buffer().get_text(start, end)
 		self.frame.clip.set_text(log)
 		
+	def OnCopyRoomLog(self, widget):
+		bound = self.RoomLog.get_buffer().get_selection_bounds()
+		if bound is not None and len(bound) == 2:
+			start, end = bound
+			log = self.RoomLog.get_buffer().get_text(start, end)
+			self.frame.clip.set_text(log)
+			
+	def OnCopyChatLog(self, widget):
+		bound = self.ChatScroll.get_buffer().get_selection_bounds()
+		if bound is not None and len(bound) == 2:
+			start, end = bound
+			log = self.ChatScroll.get_buffer().get_text(start, end)
+			self.frame.clip.set_text(log)
+		
+	def OnCopyAllChatLog(self, widget):
+		start, end = self.ChatScroll.get_buffer().get_bounds()
+		log = self.ChatScroll.get_buffer().get_text(start, end)
+		self.frame.clip.set_text(log)
+		
+	def OnClearChatLog(self, widget):
+		self.ChatScroll.get_buffer().set_text("")
+
 	def OnClearRoomLog(self, widget):
 		self.RoomLog.get_buffer().set_text("")
-
+	
 	def OnTickerClicked(self, widget, event):
 		if event.button != 1:
 			return False
