@@ -32,7 +32,8 @@ class NowPlaying:
 			self.NowPlaying.add_accel_group(self.accel_group)
 			#self.NowPlaying.show()
 			self.NowPlaying.connect("destroy", self.quit)
-        		self.NowPlaying.connect("delete_event", self.quit)
+			self.NowPlaying.connect("destroy-event", self.quit)
+			self.NowPlaying.connect("delete-event", self.quit)
 	
 		self.vbox1 = gtk.VBox(False, 5)
 		self.vbox1.show()
@@ -50,7 +51,7 @@ class NowPlaying:
 		self.hbox1.set_spacing(5)
 		self.hbox1.set_border_width(3)
 		
-		self.label2 = gtk.Label(_(""))
+		self.label2 = gtk.Label()
 	
 		self.NP_infopipe = gtk.RadioButton()
 		self.NP_infopipe.set_active(False)
@@ -67,6 +68,14 @@ class NowPlaying:
 		self.NP_amarok.connect("clicked", self.OnNPPlayer)
 	
 		self.hbox1.pack_start(self.NP_amarok, False, False, 0)
+
+		self.NP_audacious = gtk.RadioButton(self.NP_infopipe)
+		self.NP_audacious.set_active(False)
+		self.NP_audacious.set_label(_("Audacious"))
+		self.NP_audacious.show()
+		self.NP_audacious.connect("clicked", self.OnNPPlayer)
+	
+		self.hbox1.pack_start(self.NP_audacious, False, False, 0)
 	
 		self.NP_mpd = gtk.RadioButton(self.NP_infopipe)
 		self.NP_mpd.set_active(False)
@@ -172,19 +181,7 @@ class NowPlaying:
 		self.NPFormat_List = gtk.ListStore(gobject.TYPE_STRING)
 		self.NPFormat = gtk.ComboBoxEntry()
 		self.NPFormat.show()
-		self.defaultlist = [ "$n", "$a - $t", "[$a] $t", "Now $s: [$a] $t", "Now $s: $n", "$a - $b - $t", "$a - $b - $t ($l/$rKBps) from $y $c" ]
-		if self.frame.np.config.sections["players"]["npformat"] != "":
-			self.NPFormat.child.set_text(self.frame.np.config.sections["players"]["npformat"])
-		if self.frame.np.config.sections["players"]["npformatlist"] != []:
-			if self.frame.np.config.sections["players"]["npformat"] == "":
-				self.NPFormat.child.set_text(self.frame.np.config.sections["players"]["npformatlist"][0])
-			for item in self.frame.np.config.sections["players"]["npformatlist"]:
-				self.NPFormat_List.append([item])
-		if self.frame.np.config.sections["players"]["npformat"] == "":
-			self.NPFormat.child.set_text(self.defaultlist[0])
-
-		for item in self.defaultlist:
-			self.NPFormat_List.append([item])
+		
 		
 
 	
@@ -204,7 +201,7 @@ class NowPlaying:
 		self.label6.set_line_wrap(False)
 		self.label6.show()
 		
-		self.label7 = gtk.Label(_(""))
+		self.label7 = gtk.Label()
 		self.label7.set_alignment(0, 0.5)
 		self.label7.set_padding(5, 5)
 		self.label7.set_line_wrap(False)
@@ -259,12 +256,28 @@ class NowPlaying:
 		self.hbox3.pack_end(self.NPTest, False, False, 0)
 	
 		self.vbox1.pack_start(self.hbox3, False, True, 0)
+		self.defaultlist = [ "$n", "$a - $t", "[$a] $t", "Now $s: [$a] $t", "Now $s: $n", "$a - $b - $t", "$a - $b - $t ($l/$rKBps) from $y $c" ]
 		self.title_clear()
+		
 		self.OnNPPlayer(None)
 	
 		if create:
 			self.NowPlaying.add(self.vbox1)
 		
+		if self.frame.np.config.sections["players"]["npformat"] != "":
+			self.NPFormat.child.set_text(self.frame.np.config.sections["players"]["npformat"])
+		if self.frame.np.config.sections["players"]["npformatlist"] != []:
+			if self.frame.np.config.sections["players"]["npformat"] == "":
+				self.NPFormat.child.set_text(str(self.frame.np.config.sections["players"]["npformatlist"][0]))
+			for item in self.frame.np.config.sections["players"]["npformatlist"]:
+				self.NPFormat_List.append([item])
+		if self.frame.np.config.sections["players"]["npformat"] == "":
+			self.NPFormat.child.set_text(str(self.defaultlist[0]))
+
+		
+		for item in self.defaultlist:
+			self.NPFormat_List.append([str(item)])
+		# Set the active radio button
 		self.SetPlayer(self.frame.np.config.sections["players"]["npplayer"])
 		
 	def title_clear(self):
@@ -276,6 +289,8 @@ class NowPlaying:
 			self.NP_infopipe.set_active(1)
 		elif player == "amarok":
 			self.NP_amarok.set_active(1)
+		elif player == "audacious":
+			self.NP_audacious.set_active(1)
 		elif player == "mpd":
 			self.NP_mpd.set_active(1)
 		#elif player == "mp3blaster":
@@ -321,6 +336,9 @@ class NowPlaying:
 		if self.NP_amarok.get_active():
 			self.label2.set_text(_("$n Now Playing\n$l Length\n$r Bitrate\n$s Status\n$t Title\n$a Artist\n$b Album\n$c Comment\n$k Track Number\n$y Year\n$f Filename"))
 			set = 1
+		if self.NP_audacious.get_active():
+			self.label2.set_text(_("$n Now Playing\n$l Length\n$r Bitrate\n$s Status\n$t Title\n$a Artist\n$b Album\n$c Comment\n$k Track Number\n$y Year\n$f Filename"))
+			set = 1
 		if self.NP_rhythmbox.get_active():
 			self.label2.set_text(_("$n Now Playing\n$l Length\n$r Bitrate\n$s Status\n$t Title\n$a Artist\n$b Album\n$c Comment\n$k Track Number\n$y Year\n$f Filename (URI)"))
 			set = 1
@@ -334,13 +352,14 @@ class NowPlaying:
 		else:
 			self.NPCommand.set_sensitive(False)
 		if not set:
-			self.label2.set_text(_(""))
+			self.label2.set_text("")
 			
 	def OnNPCancel(self, widget):
 		self.quit(None)
 		
-	def quit(self, widget):
+	def quit(self, widget, s=None):
 		self.NowPlaying.hide()
+		return True
 		
 	def OnNPTest(self, widget):
 		self.DisplayNowPlaying(None, test=1)
@@ -350,6 +369,8 @@ class NowPlaying:
 			player = "infopipe"
 		elif self.NP_amarok.get_active():
 			player = "amarok"
+		elif self.NP_audacious.get_active():
+			player = "audacious"
 		elif self.NP_mpd.get_active():
 			player = "mpd"
 		#elif self.NP_mp3blaster.get_active():
@@ -373,6 +394,8 @@ class NowPlaying:
 			result = self.xmms()
 		elif self.NP_amarok.get_active():
 			result = self.amarok()
+		elif self.NP_audacious.get_active():
+			result = self.audacious()
 		elif self.NP_mpd.get_active():
 			result = self.mpd()
 		#elif self.NP_mp3blaster.get_active():
@@ -412,9 +435,6 @@ class NowPlaying:
 		w = gtk.Label(_("(custom widget: %s)") % id)
 		return w
 	
-	def audacious(self):
-		return None
-	
 	def bmpx(self):
 		try:
 			import dbus
@@ -493,7 +513,7 @@ class NowPlaying:
 		if "$b" in slist:
 			output = self.amarok_command("album")
 			if output:
-				self.title["length"] = output
+				self.title["album"] = output
 		if "$c" in slist:
 			output = self.amarok_command("comment")
 			if output: self.title["comment"] = output
@@ -502,7 +522,8 @@ class NowPlaying:
 			if output: self.title["track"] = output
 		if "$y" in slist:
 			output = self.amarok_command("year")
-			if output: self.title["year"] = output
+			if output and not output == "0":
+				self.title["year"] = output
 		if "$r" in slist:
 			output = self.amarok_command("bitrate")
 			if output: self.title["bitrate"] = output
@@ -529,7 +550,54 @@ class NowPlaying:
 		if output == 'call failed':
 			output = None
 		return output
+
+	def audacious(self):
+		slist = self.NPFormat.child.get_text()
 		
+		if "$n" in slist:
+			artist = self.audacious_command('current-song-tuple-data', 'performer')
+			title = self.audacious_command('current-song-tuple-data', 'track_name')
+			if artist and title: self.title["nowplaying"] = artist + ' - ' + title
+		if "$t" in slist:
+			output = self.audacious_command('current-song-tuple-data', 'track_name')
+			if output: self.title["title"] = output
+		if "$l" in slist:
+			output = self.audacious_command('current-song-length')
+			if output: self.title["length"] = output
+		if "$a" in slist:
+			output = self.audacious_command('current-song-tuple-data', 'performer')
+			if output: self.title["artist"] = output
+		if "$b" in slist:
+			output = self.audacious_command('current-song-tuple-data', 'album_name')
+			if output: self.title["album"] = output
+		if "$c" in slist:
+			output = self.audacious_command('current-song-tuple-data', 'comment')
+			if output: self.title["comment"] = output
+		if "$k" in slist:
+			output = self.audacious_command('current-song-tuple-data', 'track_number')
+			if output: self.title["track"] = output
+		if "$y" in slist:
+			output = self.audacious_command('current-song-tuple-data', 'year')
+			if output and not output == "0":
+				self.title["year"] = output
+		if "$r" in slist:
+			output = self.audacious_command('current-song-bitrate-kbps')
+			if output: self.title["bitrate"] = output
+		if "$f" in slist:
+			path = self.audacious_command('current-song-tuple-data', 'file_path')
+			ext = self.audacious_command('current-song-tuple-data', 'file_ext')
+			if path and ext: self.title["filename"] = path + ext
+		if "$s" in slist:
+			output = self.audacious_command('playback-status')
+			if output: self.title["status"] = output
+		return 1
+		
+	def audacious_command(self, command, subcommand = ''):
+		output = commands.getoutput("audtool %s %s" % (command, subcommand)).split('\n')[0]
+		if output == 'call failed':
+			output = None
+		return output
+
 	def mp3blaster(self):
 		return None
 	
@@ -546,7 +614,7 @@ class NowPlaying:
 			self.rbshell = dbus.Interface(rbshellobj, 'org.gnome.Rhythmbox.Shell')
 			rbplayerobj = bus.get_object('org.gnome.Rhythmbox', '/org/gnome/Rhythmbox/Player')
 			rbplayer = dbus.Interface(rbplayerobj, 'org.gnome.Rhythmbox.Player')
- 		except Exception, e:
+		except Exception, e:
 			print "NowPlaying Error opening Rhythmbox interface on dbus", e
 			return None
 
