@@ -3,9 +3,9 @@
 import gtk
 import gobject
 import os
-
+import pango
 from nicotine_glade import PrivateChatTab
-from utils import AppendLine, IconNotebook, PopupMenu, WriteLog, expand_alias, EncodingsMenu, SaveEncoding
+from utils import AppendLine, IconNotebook, PopupMenu, WriteLog, expand_alias, EncodingsMenu, SaveEncoding, fixpath
 from chatrooms import GetCompletion
 from pynicotine import slskmessages
 
@@ -30,7 +30,7 @@ class PrivateChats(IconNotebook):
 			if tab.Main == page:
 				gobject.idle_add(tab.ChatLine.grab_focus)
 				# Remove hilite if selected tab belongs to a user in the hilite list
-				if user in self.frame.tray_status["hilites"]["private"]:
+				if user in self.frame.TrayApp.tray_status["hilites"]["private"]:
 					self.frame.ClearNotification("private", tab.user)
 		
 	def GetUserStatus(self, msg):
@@ -45,6 +45,7 @@ class PrivateChats(IconNotebook):
 			tab = PrivateChat(self, user)
 			self.users[user] = tab
 			self.append_page(tab.Main, user, tab.OnClose)
+			self.frame.np.queue.put(slskmessages.AddUser(user))
 			self.frame.np.queue.put(slskmessages.GetUserStatus(user))
 		if direction:
 			if self.get_current_page() != self.page_num(self.users[user].Main):
@@ -75,18 +76,26 @@ class PrivateChats(IconNotebook):
 
 	def UpdateColours(self):
 		for chat in self.users.values():
-			chat.UpdateColours()
+			chat.ChangeColours()
 
 	def RemoveTab(self, tab):
 		self.remove_page(tab.Main)
-		if tab.user in self.frame.tray_status["hilites"]["private"]:
+		if tab.user in self.frame.TrayApp.tray_status["hilites"]["private"]:
 			self.frame.ClearNotification("private", tab.user)
 		del self.users[tab.user]
+		if tab.user in self.frame.np.config.sections["privatechat"]["users"]:
+			self.frame.np.config.sections["privatechat"]["users"].remove(tab.user)
 
 	def Login(self):
 		self.connected = 1
 		for user in self.users:
 			self.users[user].Login()
+			
+		if self.frame.np.config.sections["privatechat"]["store"]:
+			self.frame.np.config.sections["privatechat"]["users"].sort()
+			for user in self.frame.np.config.sections["privatechat"]["users"]:
+				if user not in self.users.keys():
+					self.SendMessage(user, None, 1)
 					
 	def ConnClose(self):
 		self.connected = 0
@@ -117,46 +126,72 @@ class PrivateChat(PrivateChatTab):
 			self.Elist[item[1]] = self.EncodingStore.append([item[1], item[0] ])
 			if self.encoding == item[1]:
 				self.Encoding.set_active_iter(self.Elist[self.encoding])
-		
+		if self.frame.SEXY and self.frame.np.config.sections["ui"]["spellcheck"]:
+			import sexy
+			self.hbox5.remove(self.ChatLine)
+			self.ChatLine.destroy()
+			self.ChatLine = sexy.SpellEntry()
+			self.ChatLine.show()
+			self.ChatLine.connect("activate", self.OnEnter)
+			self.ChatLine.connect("key_press_event", self.OnKeyPress)
+			self.hbox5.pack_start(self.ChatLine)
+			self.hbox5.reorder_child(self.ChatLine, 0)
+			
 		self.Log.set_active(self.frame.np.config.sections["logging"]["privatechat"])
 
-		self.UpdateColours()
-
 		if self.frame.translux:
 			self.tlux_chat = lambda: self.ChatScroll.get_window(gtk.TEXT_WINDOW_TEXT)
 			self.frame.translux.subscribe(self.ChatScroll, self.tlux_chat)
 			self.ChatScroll.get_parent().get_vadjustment().connect("value-changed", lambda *args: self.ChatScroll.queue_draw())
 			self.ChatScroll.get_parent().get_hadjustment().connect("value-changed", lambda *args: self.ChatScroll.queue_draw())
-
-		self.popup_menu = popup = PopupMenu(self.frame)
+		
+		self.popup_menu_user = popup = PopupMenu(self.frame)
 		popup.setup(
-			("#" + _("Close"), self.OnClose, gtk.STOCK_CANCEL),
-			("", None),
 			("#" + _("Show IP a_ddress"), popup.OnShowIPaddress, gtk.STOCK_NETWORK),
 			("#" + _("Get user i_nfo"), popup.OnGetUserInfo, gtk.STOCK_DIALOG_INFO),
 			("#" + _("Brow_se files"), popup.OnBrowseUser, gtk.STOCK_HARDDISK),
 			("#" + _("Gi_ve privileges"), popup.OnGivePrivileges, gtk.STOCK_JUMP_TO),
+			("#" + _("Client Version"), popup.OnVersion, gtk.STOCK_ABOUT ),
+			("", None),
 			("$" + _("Add user to list"), popup.OnAddToList),
 			("$" + _("Ban this user"), popup.OnBanUser),
 			("$" + _("Ignore this user"), popup.OnIgnoreUser),
-			("#" + _("Client Version"), popup.OnVersion, gtk.STOCK_ABOUT ),
+		)
+		popup.set_user(user)
+		self.popup_menu = popup = PopupMenu(self.frame)
+		popup.setup(
+			("USERMENU", _("User"), self.popup_menu_user, self.OnPopupMenuUser),
+			("", None),
+			("#" + _("Find"), self.OnFindChatLog, gtk.STOCK_FIND),
+			("", None),
+			("#" + _("Copy"), self.OnCopyChatLog, gtk.STOCK_COPY),
+			("#" + _("Copy All"), self.OnCopyAllChatLog, gtk.STOCK_COPY),
+			("", None),
+			("#" + _("Clear log"), self.OnClearChatLog, gtk.STOCK_CLEAR),
+			("", None),
+			("#" + _("Close"), self.OnClose, gtk.STOCK_CANCEL),
+			
 			
 		)
 		popup.set_user(user)
 		self.ChatScroll.connect("button_press_event", self.OnPopupMenu)
 
-		log = os.path.join(self.frame.np.config.sections["logging"]["logsdir"], self.user.replace(os.sep, "-") + ".log")
+		self.UpdateColours()
+		
+		# Read log file
+		log = os.path.join(self.frame.np.config.sections["logging"]["logsdir"], fixpath(self.user.replace(os.sep, "-")) + ".log")
 		try:
 			f = open(log, "r")
 			d = f.read()
 			f.close()
 			s = d.split("\n")
 			for l in s[-8:-1]:
-				AppendLine(self.ChatScroll, l + "\n", self.tag_hilite, "")
+				AppendLine(self.ChatScroll, l + "\n", self.tag_hilite, "", username=self.user, usertag=self.tag_hilite)
+				
 		except IOError, e:
 			pass
 
-				
+		
 	def destroy(self):
 		if self.frame.translux:
 			self.frame.translux.unsubscribe(self.tlux_chat)
@@ -164,22 +199,46 @@ class PrivateChat(PrivateChatTab):
 		
 	def Login(self):
 		AppendLine(self.ChatScroll, _("--- reconnected ---"), self.tag_hilite, "%c")
-		
+		self.ChangeColours()
 	def ConnClose(self):
 		AppendLine(self.ChatScroll, _("--- disconnected ---"), self.tag_hilite, "%c")
 		self.offlinemessage = 0
-		
+		self.ChangeColours()
 	def OnPopupMenu(self, widget, event):
 		if event.button != 3:
 			return
-		items = self.popup_menu.get_children()
-		items[6].set_active(self.user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
-		items[7].set_active(self.user in self.frame.np.config.sections["server"]["banlist"])
-		items[8].set_active(self.user in self.frame.np.config.sections["server"]["ignorelist"])
+		
 		self.popup_menu.popup(None, None, None, event.button, event.time)
 		self.ChatScroll.emit_stop_by_name("button_press_event")
 		return True
-
+	
+	def OnPopupMenuUser(self, widget):
+		items = self.popup_menu_user.get_children()
+		
+		items[6].set_active(self.user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
+		items[7].set_active(self.user in self.frame.np.config.sections["server"]["banlist"])
+		items[8].set_active(self.user in self.frame.np.config.sections["server"]["ignorelist"])
+	
+		return True
+	
+	def OnFindChatLog(self, widget):
+		self.frame.OnFindTextview(widget, self.ChatScroll)
+		
+	def OnCopyChatLog(self, widget):
+		bound = self.ChatScroll.get_buffer().get_selection_bounds()
+		if bound is not None and len(bound) == 2:
+			start, end = bound
+			log = self.ChatScroll.get_buffer().get_text(start, end)
+			self.frame.clip.set_text(log)
+		
+	def OnCopyAllChatLog(self, widget):
+		start, end = self.ChatScroll.get_buffer().get_bounds()
+		log = self.ChatScroll.get_buffer().get_text(start, end)
+		self.frame.clip.set_text(log)
+		
+	def OnClearChatLog(self, widget):
+		self.ChatScroll.get_buffer().set_text("")
+	
 	def ShowMessage(self, text, status=None):
 		if text[:4] == "/me ":
 			line = "* %s %s" % (self.user, text[4:])
@@ -188,7 +247,7 @@ class PrivateChat(PrivateChatTab):
 			line = "[%s] %s" % (self.user, text)
 			tag = self.tag_remote
 		line = self.frame.np.decode(line, self.encoding)
-		AppendLine(self.ChatScroll, line, tag, "%c")
+		AppendLine(self.ChatScroll, line, tag, "%c", username=self.user, usertag=self.tag_my_username)
 		if self.Log.get_active():
 			self.logfile = WriteLog(self.logfile, self.frame.np.config.sections["logging"]["logsdir"], self.user, line)
 		
@@ -202,10 +261,11 @@ class PrivateChat(PrivateChatTab):
 			
 
 	def SendMessage(self, text):
-
+		my_username = self.frame.np.config.sections["server"]["login"]
 		if text[:4] == "/me ":
-			line = "* %s %s" % (self.frame.np.config.sections["server"]["login"], text[4:])
-			tag = self.tag_me
+			line = "* %s %s" % (my_username, text[4:])
+			usertag = tag = self.tag_me
+			message = self.frame.np.decode(line, self.encoding)
 		else:
 			
 			if text == "\x01VERSION\x01":
@@ -213,14 +273,21 @@ class PrivateChat(PrivateChatTab):
 			else:
 				line = text
 			tag = self.tag_local
-			
-		AppendLine(self.ChatScroll, self.frame.np.decode(line, self.encoding), tag, "%c")
+			usertag = self.tag_my_username
+			message = "[" + my_username + "] " + self.frame.np.decode(line, self.encoding)
+		
+		AppendLine(self.ChatScroll, message, tag, "%c", username=my_username, usertag=usertag)
 		if self.Log.get_active():
-			self.logfile = WriteLog(self.logfile, self.frame.np.config.sections["logging"]["logsdir"], self.user, line)
-		self.frame.np.queue.put(slskmessages.MessageUser(self.user, text))
-	
+			self.logfile = WriteLog(self.logfile, self.frame.np.config.sections["logging"]["logsdir"], self.user, message)
+		
+		if self.PeerPrivateMessages.get_active():
+			# not in the soulseek protocol
+			self.frame.np.ProcessRequestToPeer(self.user, slskmessages.PMessageUser(None, my_username, text))
+		else:
+			self.frame.np.queue.put(slskmessages.MessageUser(self.user, text))
+			
 	CMDS = ["/alias ", "/unalias ", "/whois ", "/browse ", "/ip ", "/pm ", "/msg ", "/search ", "/usearch ", "/rsearch ",
-		"/bsearch ", "/add ", "/buddy ", "/rem ", "/unbuddy ", "/ban ", "/ignore ", "/unban ", "/unignore ", "/clear", "/quit", "/rescan", "/nsa", "/info", "/ctcpversion"]
+		"/bsearch ", "/add ", "/buddy ", "/rem ", "/unbuddy ", "/ban ", "/ignore ", "/unban ", "/unignore ", "/clear", "/quit", "/rescan", "/nsa", "/info", "/ctcpversion", "/join"]
 
 	def OnEnter(self, widget):
 		text = self.frame.np.encode(widget.get_text(), self.encoding)
@@ -243,6 +310,8 @@ class PrivateChat(PrivateChatTab):
 			AppendLine(self.ChatScroll, self.frame.np.config.AddAlias(realargs), None, "")
 		elif cmd in ("/unalias", "/un"):
 			AppendLine(self.ChatScroll, self.frame.np.config.Unalias(realargs), None, "")
+		elif cmd in ["/join", "/j"]:
+			self.frame.np.queue.put(slskmessages.JoinRoom(args))
 		elif cmd in ["/w", "/whois", "/info"]:
 			if args:
 				self.frame.LocalUserInfoRequest(args)
@@ -334,28 +403,135 @@ class PrivateChat(PrivateChatTab):
 			return
 		widget.set_text("")
 
+	def makecolour(self, buffer, colour):
+		color = self.frame.np.config.sections["ui"][colour]
+		if color == "":
+			color = self.backupcolor
+		else:
+			color = gtk.gdk.color_parse(color)
+		
+		font = self.frame.np.config.sections["ui"]["chatfont"]
+		tag = buffer.create_tag()
+		tag.set_property("foreground-gdk", color)
+		tag.set_property("font", font)
+		return tag
+		
 	def UpdateColours(self):
-		def makecolour(buffer, colour):
-			colour = self.frame.np.config.sections["ui"][colour]
-			font = self.frame.np.config.sections["ui"]["chatfont"]
-			if colour:
-				return buffer.create_tag(foreground = colour, font=font)
-			else:
-				return buffer.create_tag( font=font)
+		map = self.frame.MainWindow.get_style().copy()
+		self.backupcolor = map.text[gtk.STATE_NORMAL]
 
-				
 		buffer = self.ChatScroll.get_buffer()
-		self.tag_remote = makecolour(buffer, "chatremote")
-		self.tag_local = makecolour(buffer, "chatlocal")
-		self.tag_me = makecolour(buffer, "chatme")
-		self.tag_hilite = makecolour(buffer, "chathilite")
+		self.tag_remote = self.makecolour(buffer, "chatremote")
+		self.tag_local = self.makecolour(buffer, "chatlocal")
+		self.tag_me = self.makecolour(buffer, "chatme")
+		self.tag_hilite = self.makecolour(buffer, "chathilite")
+		if self.status == 1:
+			statuscolor = "useraway"
+		elif self.status == 2:
+			statuscolor = "useronline"
+		else:
+			statuscolor = "useroffline"
+		if self.chats.connected:
+			if self.frame.away:
+				self.tag_my_username = self.makecolour(buffer, "useraway")
+			else:
+				self.tag_my_username = self.makecolour(buffer, "useronline")
+		else:
+			self.tag_my_username = self.makecolour(buffer, "useroffline")
+		self.tag_username = self.makecolour(buffer, statuscolor)
+		usernamestyle = self.frame.np.config.sections["ui"]["usernamestyle"]
+		if usernamestyle == "bold":
+			self.tag_username.set_property("weight",  pango.WEIGHT_BOLD)
+			self.tag_my_username.set_property("weight",  pango.WEIGHT_BOLD)
+		else:
+			self.tag_username.set_property("weight",  pango.WEIGHT_NORMAL)
+			self.tag_my_username.set_property("weight",  pango.WEIGHT_NORMAL)
+		if usernamestyle == "italic":
+			self.tag_username.set_property("style",  pango.STYLE_ITALIC)
+			self.tag_my_username.set_property("style",  pango.STYLE_ITALIC)
+		else:
+			self.tag_username.set_property("style",  pango.STYLE_NORMAL)
+			self.tag_my_username.set_property("style",  pango.STYLE_NORMAL)
+		if usernamestyle == "underline":
+			self.tag_username.set_property("underline", pango.UNDERLINE_SINGLE)
+			self.tag_my_username.set_property("underline", pango.UNDERLINE_SINGLE)
+		else:
+			self.tag_username.set_property("underline", pango.UNDERLINE_NONE)
+			self.tag_my_username.set_property("underline", pango.UNDERLINE_NONE)
+		self.frame.SetTextBG(self.ChatScroll)
+		self.frame.SetTextBG(self.ChatLine)
+		
+	def changecolour(self, tag, colour):
+		if self.frame.np.config.sections["ui"].has_key(colour):
+			color = self.frame.np.config.sections["ui"][colour]
+		else:
+			color = ""
+		font = self.frame.np.config.sections["ui"]["chatfont"]
 
+		if color == "":
+			color = self.backupcolor
+		else:
+			color = gtk.gdk.color_parse(color)
+		tag.set_property("foreground-gdk", color)
+		tag.set_property("font", font)
+		if colour in ["useraway", "useronline", "useroffline"]:
+			usernamestyle = self.frame.np.config.sections["ui"]["usernamestyle"]
+			if usernamestyle == "bold":
+				tag.set_property("weight",  pango.WEIGHT_BOLD)
+			else:
+				tag.set_property("weight",  pango.WEIGHT_NORMAL)
+			if usernamestyle == "italic":
+				tag.set_property("style",  pango.STYLE_ITALIC)
+			else:
+				tag.set_property("style",  pango.STYLE_NORMAL)
+			if usernamestyle == "underline":
+				tag.set_property("underline", pango.UNDERLINE_SINGLE)
+			else:
+				tag.set_property("underline", pango.UNDERLINE_NONE)
+	
+			
+	def ChangeColours(self):
+		map = self.ChatScroll.get_style().copy()
+		self.backupcolor = map.text[gtk.STATE_NORMAL]
+		
+		self.changecolour(self.tag_remote, "chatremote")
+		self.changecolour(self.tag_local, "chatlocal")
+		self.changecolour(self.tag_me, "chatme")
+		self.changecolour(self.tag_hilite, "chathilite")
+		color = self.getUserStatusColor(self.status)
+		self.changecolour(self.tag_username, color)
+		self.frame.SetTextBG(self.ChatScroll)
+		self.frame.SetTextBG(self.ChatLine)
+		
+		if self.chats.connected:
+			if self.frame.away:
+				self.changecolour(self.tag_my_username, "useraway")
+			else:
+				self.changecolour(self.tag_my_username, "useronline")
+		else:
+			self.changecolour(self.tag_my_username, "useroffline")
+	def getUserStatusColor(self, status):
+		if status == 1:
+			color = "useraway"
+		elif status == 2:
+			color = "useronline"
+		else:
+			color = "useroffline"
+		return color
+		
 	def GetUserStatus(self, status):
 		if status == self.status:
 			return
+		
+		
 		self.status = status
-		line = "* " + ["User %s is offline", "User %s is away", "User %s is online"][status] % self.user
-		AppendLine(self.ChatScroll, line, self.tag_hilite, "%c")
+
+		color = self.getUserStatusColor(self.status)
+
+		self.changecolour(self.tag_username, color)
+		
+		#line = "* " + ["User %s is offline", "User %s is away", "User %s is online"][status] % self.user
+		#AppendLine(self.ChatScroll, line, self.tag_hilite, "%c")
 	
 	def OnClose(self, widget):
 		if self.logfile is not None:
