@@ -7,20 +7,148 @@ import re
 import sre_constants
 import locale
 import string
+import random
 
 from pynicotine import slskmessages
 
 from nicotine_glade import SearchTab
-from utils import InitialiseColumns, PopupMenu, FastListModel, Humanize
+from utils import InitialiseColumns, PopupMenu, FastListModel, Humanize, PressHeader
 from dirchooser import ChooseDir
-
+from entrydialog import *
 from pynicotine.utils import _
+from utils import InputDialog
 
+class WishList( gtk.Dialog):
+	def __init__(self, frame):
+		gtk.Dialog.__init__(self)
+		self.set_title(_("Nicotine+ Wishlist"))
+		self.set_icon(frame.images["n"])
+		self.connect("destroy", self.quit)
+		self.connect("destroy-event", self.quit)
+		self.connect("delete-event", self.quit)
+		self.connect("delete_event", self.quit)
+		self.nicotine = frame
+		self.set_size_request(250, 250)
+		self.mainHbox = gtk.HBox(False, 5)
+		self.mainHbox.show()
+		
+		
+
+		self.WishLabel = gtk.Label(_("Search Wishlist"))
+		self.WishLabel.set_padding(0, 0)
+		self.WishLabel.set_line_wrap(False)
+		self.WishLabel.show()
+		self.vbox.pack_start(self.WishLabel, False, True, 0)
+
+		self.WishScrollWin = gtk.ScrolledWindow()
+		self.WishScrollWin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+		self.WishScrollWin.show()
+		self.WishScrollWin.set_shadow_type(gtk.SHADOW_IN)
+
+		self.WishlistView = gtk.TreeView()
+		self.WishlistView.show()
+		self.WishlistView.set_headers_visible(False)
+		self.WishScrollWin.add(self.WishlistView)
+		
+		self.mainHbox.pack_start(self.WishScrollWin, True, True, 0)
+		self.mainVbox = gtk.VBox(False, 5)
+		self.mainHbox.pack_start(self.mainVbox, False, False)
+		self.mainVbox.show()
+		self.mainVbox.set_spacing(5)
+		
+		self.AddWishButton = self.nicotine.CreateIconButton(gtk.STOCK_ADD, "stock", self.OnAddWish, _("Add..."))
+		self.mainVbox.pack_start(self.AddWishButton, False, False, 0)
+
+
+		self.RemoveWishButton = self.nicotine.CreateIconButton(gtk.STOCK_REMOVE, "stock", self.OnRemoveWish, _("Remove"))
+		self.mainVbox.pack_start(self.RemoveWishButton, False, False, 0)
+
+
+		self.ClearWishesButton = self.nicotine.CreateIconButton(gtk.STOCK_CLEAR, "stock", self.OnClearWishes, _("Clear"))
+
+		
+		self.mainVbox.pack_start(self.ClearWishesButton, False, False, 0)
+
+		self.CloseButton = self.nicotine.CreateIconButton(gtk.STOCK_CLOSE, "stock", self.quit, _("Close"))
+		self.mainVbox.pack_end(self.CloseButton, False, False, 0)
+		self.vbox.pack_start(self.mainHbox, True, True, 0)
+		
+
+		self.store = gtk.ListStore(gobject.TYPE_STRING)
+		column = gtk.TreeViewColumn(_("Wishs"), gtk.CellRendererText(), text = 0)
+		self.WishlistView.append_column(column)
+		self.WishlistView.set_model(self.store)
+		self.WishlistView.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+		column.set_sort_column_id(0)
+		self.store.set_sort_column_id(0, gtk.SORT_ASCENDING)
+		self.wishes = {}
+		for wish in self.nicotine.np.config.sections["server"]["autosearch"]:
+			self.wishes[wish] = self.store.append([wish])
+		
+	def OnAddWish(self, widget):
+		wish = InputDialog(self.vbox.get_toplevel(), _("Add Wish..."), _("Wish:") )
+		if wish and wish not in self.wishes:
+			self.wishes[wish] = self.store.append([wish])
+			self.nicotine.searches.NewWish(wish)
+	
+	def _RemoveWish(self, model, path, iter, l):
+		l.append(iter)
+		
+	def removeWish(self, wish):
+		if self.wishes.has_key(wish):
+			self.store.remove(self.wishes[wish])
+			del self.wishes[wish]
+		if wish in self.nicotine.np.config.sections["server"]["autosearch"]:
+			self.nicotine.np.config.sections["server"]["autosearch"].remove(wish)
+			for number, search in self.nicotine.searches.searches.items():
+				if search[1] == wish and search[4] == 1:
+					search[4] = 0
+					self.nicotine.searches.searches[number] = search
+					if search[2] is not None:
+				
+						search[2].RememberCheckButton.set_active(False)
+					break
+
+		
+	def addWish(self, wish):
+		if wish and wish not in self.wishes:
+			self.wishes[wish] = self.store.append([wish])
+		
+	def OnRemoveWish(self, widget):
+		iters = []
+		self.WishlistView.get_selection().selected_foreach(self._RemoveWish, iters)
+		for iter in iters:
+			wish = self.store.get_value(iter, 0)
+			self.removeWish(wish)
+	
+
+	def OnClearWishes(self, widget):
+		self.wishes = {}
+		self.store.clear()
+		for number, search in self.nicotine.searches.searches.items():
+			search[4] = 0
+			self.nicotine.searches.searches[number] = search
+			if search[2] is not None:
+				search[2].RememberCheckButton.set_active(False)
+
+		self.nicotine.np.config.sections["server"]["autosearch"] = []
+		self.nicotine.np.config.writeConfig()
+		
+	def quit(self, w=None, event=None):
+
+		self.hide()
+		return True
+
+	def Toggle(self, widget):
+		if self.get_property("visible"):
+			self.hide()
+		else:
+			self.show()
 class Searches:
 	def __init__(self, frame):
 		self.frame = frame
 		self.interval = 0
-		self.searchid = 0
+		self.searchid = int(random.random() * (2**31-1))
 		self.searches = {}
 		self.timer = None
 		self.disconnected = 0
@@ -35,14 +163,17 @@ class Searches:
 				templist.append(i)
 		for i in templist:
 			self.frame.combo1.append_text(i)
+		self.WishListDialog = WishList(frame)
+		self.frame.WishList.connect("clicked", self.WishListDialog.Toggle)
 		
 	def SetInterval(self, msg):
-		self.interval = 1000
-		
+		self.interval = msg.num
 		if not self.disconnected:
+			# Create wishlist searches (without tabs)
 			for term in self.frame.np.config.sections["server"]["autosearch"]:
-				self.CreateTab(self.searchid, term, 0, 1)
-				self.searchid += 1
+
+				self.searches[self.searchid] = [self.searchid, term, None, 0, True]
+				self.searchid = (self.searchid + 1) % (2**31)
 		
 		self.OnAutoSearch()
 		self.timer = gobject.timeout_add(self.interval*1000, self.OnAutoSearch)
@@ -54,6 +185,7 @@ class Searches:
 			self.timer = None
 	
 	def OnAutoSearch(self, *args):
+		# Wishlists supported by server?
 		if self.interval == 0:
 			return False
 		
@@ -61,16 +193,13 @@ class Searches:
 		if not searches:
 			return True
 		
-		term = searches.pop()
-		searches.insert(0, term)
-		
-		for i in self.searches.values():
-			if i[1] == term and i[4]:
-				if i[2] == None:
-					break
-				self.DoGlobalSearch(i[0], term)
-				break
-		
+		# Search for a maximum of 3 items at each search interval
+		for term in searches[0:3]:
+			for i in self.searches.values():
+				if i[1] == term and i[4]:
+					self.DoWishListSearch(i[0], term)
+					oldsearch = searches.pop()
+					searches.insert(0, oldsearch)
 		return True
 	
 	def OnClearSearchHistory(self):
@@ -81,19 +210,46 @@ class Searches:
 	
 	def OnSearch(self):
 		text = self.frame.SearchEntry.get_text().strip()
-		self.frame.SearchEntry.set_text("")
+		
 		if not text:
 			return
-
-		if self.frame.GlobalRadio.get_active():
+		users = []
+		room = None
+		if self.frame.SearchMethod.get_active_text() == _("Global"):
 			mode = 0
-		elif self.frame.RoomsRadio.get_active():
+		elif self.frame.SearchMethod.get_active_text() == _("Rooms"):
 			mode = 1
-		else:
+			name = self.frame.RoomSearchCombo.child.get_text()
+			# Space after Joined Rooms is important, so it doesn't conflict
+			# with any possible real room
+			if name != _("Joined Rooms ") and not name.isspace():
+				room = name
+		elif self.frame.SearchMethod.get_active_text() == _("Buddies"):
 			mode = 2
-		self.DoSearch(text, mode)
+		elif self.frame.SearchMethod.get_active_text() == _("User"):
+			mode = 3
+			user = self.frame.UserSearchCombo.child.get_text()
+			if user != "" and not user.isspace():
+				users = [user]
+			else:
+				return
+		else:
+			mode = 0
+			
+		self.DoSearch(text, mode, users, room)
+		self.frame.SearchEntry.set_text("")
 		
-	def DoSearch(self, text, mode, users = []):
+	def NewWish(self, wish):
+		if wish in self.frame.np.config.sections["server"]["autosearch"]:
+			return
+		self.frame.np.config.sections["server"]["autosearch"].append(wish)
+			
+		self.searchid += 1
+		self.searches[self.searchid] = [self.searchid, wish, None, 0, True]
+			
+		self.DoWishListSearch(self.searchid, wish)
+		
+	def DoSearch(self, text, mode, users = [], room = None):
 		items = self.frame.np.config.sections["searches"]["history"]
 		if text in items:
 			items.remove(text)
@@ -110,24 +266,34 @@ class Searches:
 		for i in templist:
 			self.frame.combo1.append_text(i)
 			
-		self.CreateTab(self.searchid, text, mode)
+		search = self.CreateTab(self.searchid, text, mode)
+		if search[2] is not None:
+			self.frame.SearchNotebook.set_current_page(self.frame.SearchNotebook.page_num(search[2].vbox7))
 		text = self.frame.np.encode(text)
 		if mode == 0:
 			self.DoGlobalSearch(self.searchid, text)
 		elif mode == 1:
-			self.DoRoomsSearch(self.searchid, text)
+			self.DoRoomsSearch(self.searchid, text, room)
 		elif mode == 2:
 			self.DoBuddiesSearch(self.searchid, text)
 		else:
-			self.DoPeerSearch(self.searchid, text, users)
+			if users != [] and users[0] != '':
+				self.DoPeerSearch(self.searchid, text, users)
 		self.searchid += 1
 		
 	def DoGlobalSearch(self, id, text):
 		self.frame.np.queue.put(slskmessages.FileSearch(id, text))
-	
-	def DoRoomsSearch(self, id, text):
-		for room in self.frame.chatrooms.roomsctrl.joinedrooms.keys():
+		
+	def DoWishListSearch(self, id, text):
+		self.frame.np.queue.put(slskmessages.WishlistSearch(id, text))
+		
+	def DoRoomsSearch(self, id, text, room = None):
+
+		if room != None:
 			self.frame.np.queue.put(slskmessages.RoomSearch(room, id, text))
+		else:
+			for room in self.frame.chatrooms.roomsctrl.joinedrooms.keys():
+				self.frame.np.queue.put(slskmessages.RoomSearch(room, id, text))
 
 
 	def DoBuddiesSearch(self, id, text):
@@ -170,6 +336,7 @@ class Searches:
 			self.frame.np.config.sections["server"]["autosearch"].remove(search[1])
 			self.frame.np.config.writeConfig()
 		search[4] = 0
+		self.WishListDialog.removeWish(search[1])
 		
 	def RemoveTab(self, tab):
 		if self.searches.has_key(tab.id):
@@ -189,7 +356,24 @@ class Searches:
 		self.frame.np.config.sections["server"]["autosearch"].append(i[1])
 		self.frame.np.config.writeConfig()
 		i[4] = 1
-	
+		self.WishListDialog.addWish(i[1])
+		
+	def UpdateColours(self):
+		for id in self.searches.values():
+			id[2].ChangeColours()
+			
+	def saveColumns(self):
+		page_num = self.frame.SearchNotebook.get_current_page()
+		if page_num is not None:
+			page = self.frame.SearchNotebook.get_nth_page(page_num)
+			for name, search in self.searches.items():
+				if search[2] is None:
+					continue
+				if search[2].vbox7 == page:
+					search[2].saveColumns()
+					break
+
+		
 class SearchTreeModel(FastListModel):
 	COLUMNS = 14
 	COLUMN_TYPES = [gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING,
@@ -292,7 +476,7 @@ class SearchTreeModel(FastListModel):
 			return False
 		if filters[3] and not self.checkDigit(filters[3], row[10], False):
 			return False
-		if filters[4] and row[6] != "Y":
+		if filters[4] and row[6] != _("Y"):
 			return False
 		if filters[5]:
 			for cc in filters[5]:
@@ -339,7 +523,7 @@ class SearchTreeModel(FastListModel):
 		for row in self.all_data:
 			if self.check_filter(row):
 				self.data.append(row)
-
+	
 class Search(SearchTab):
 	def __init__(self, searches, text, id, mode, remember):
 		SearchTab.__init__(self, False)
@@ -372,7 +556,7 @@ class Search(SearchTab):
 			self.FilterFreeSlot.set_active(filter[4])
 			if(len(filter) > 5):
 				self.FilterCountry.child.set_text(filter[5])
-			self.checkbutton1.set_active(1)
+			self.filtersCheck.set_active(1)
 
 		if mode > 0:
 			self.RememberCheckButton.set_sensitive(False)
@@ -384,7 +568,7 @@ class Search(SearchTab):
 		self.ResultsList.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
 		self.ResultsList.set_property("rules-hint", True)
 		cols = InitialiseColumns(self.ResultsList,
-			["", 20, "text", self.CellDataFunc],
+			[_("Number"), 20, "text", self.CellDataFunc],
 			[_("Filename"), 250, "text", self.CellDataFunc],
 			[_("User"), 100, "text", self.CellDataFunc],
 			[_("Size"), 100, "text", self.CellDataFunc],
@@ -395,6 +579,14 @@ class Search(SearchTab):
 			[_("Length"), 50, "text", self.CellDataFunc],
 			[_("Directory"), 1000, "text", self.CellDataFunc],
 		)
+		cols[0].get_widget().hide()
+		for i in range (10):
+			
+			parent = cols[i].get_widget().get_ancestor(gtk.Button)
+			if parent:
+				parent.connect('button_press_event', PressHeader)
+			# Read Show / Hide column settings from last session
+			cols[i].set_visible(self.frame.np.config.sections["columns"]["search"][i])
 		self.ResultsList.set_model(self.resultsmodel)
 		for ix in range(len(cols)):
 			col = cols[ix]
@@ -410,6 +602,7 @@ class Search(SearchTab):
 			("#" + _("_Download file(s)"), self.OnDownloadFiles, gtk.STOCK_GO_DOWN),
 			("#" + _("Download file(s) _to..."), self.OnDownloadFilesTo, gtk.STOCK_GO_DOWN),
 			("#" + _("Download containing _folder(s)"), self.OnDownloadFolders, gtk.STOCK_GO_DOWN),
+			("#" + _("View Metadata of file(s)"), self.OnSearchMeta, gtk.STOCK_PROPERTIES),
 			("", None),
 			("#" + _("Copy _URL"), self.OnCopyURL, gtk.STOCK_COPY),
 			("#" + _("Copy folder URL"), self.OnCopyDirURL, gtk.STOCK_COPY),
@@ -428,12 +621,30 @@ class Search(SearchTab):
 		
 		self._more_results = 0
 		self.new_results = []
-	
+		self.ChangeColours()
+		
+	def ChangeColours(self):
+		self.frame.SetTextBG(self.ResultsList)
+		self.frame.SetTextBG(self.FilterIn.child)
+		self.frame.SetTextBG(self.FilterOut.child)
+		self.frame.SetTextBG(self.FilterSize.child)
+		self.frame.SetTextBG(self.FilterBitrate.child)
+		self.frame.SetTextBG(self.FilterCountry.child)
+
+	def saveColumns(self):
+		columns = []
+		for column in self.ResultsList.get_columns():
+			columns.append(column.get_visible())
+		self.frame.np.config.sections["columns"]["search"] = columns
+		
 	def SelectedResultsCallback(self, model, path, iter):
 		user = model.get_value(iter, 2)
 		fn = model.get_value(iter, 11)
+		size = model.get_value(iter, 13)
+		bitrate = model.get_value(iter, 7)
+		length = model.get_value(iter, 8)
 		
-		self.selected_results.append((user, fn))
+		self.selected_results.append((user, fn, size, bitrate, length))
 		
 		if not user in self.selected_users:
 			self.selected_users.append(user)
@@ -468,19 +679,19 @@ class Search(SearchTab):
 			act = True
 		else:
 			act = False
-		items[4].set_sensitive(act)
 		items[5].set_sensitive(act)
+		items[6].set_sensitive(act)
 
 		act = False
 		if len(self.selected_users) == 1:
 			act = True
 			user = self.selected_users[0]
 			self.popup_menu.set_user(user)
-			items[12].set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
-			items[13].set_active(user in self.frame.np.config.sections["server"]["banlist"])
-			items[14].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
+			items[13].set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
+			items[14].set_active(user in self.frame.np.config.sections["server"]["banlist"])
+			items[15].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
 		
-		for i in range(7, 14):
+		for i in range(7, 16):
 			items[i].set_sensitive(act)
 		
 		widget.emit_stop_by_name("button_press_event")
@@ -553,11 +764,47 @@ class Search(SearchTab):
 		colour = self.frame.np.config.sections["ui"][colour] or None
 		cellrenderer.set_property("foreground", colour)
 
+	def MetaBox(self, title="Meta Data", message="", data=None, modal= True):
+		win = MetaDialog( self.frame, message,  data, modal)
+		win.set_title(title)
+		win.set_icon(self.frame.images["n"])
+		win.set_default_size(300, 100)
+		win.show()
+		gtk.main()
+		return win.ret
+	
+	def SelectedResultsAllData(self, model, path, iter, data):
+		num = model.get_value(iter, 0)
+		filename = model.get_value(iter, 1)
+		user = model.get_value(iter, 2)
+		size = model.get_value(iter, 3)
+		speed = model.get_value(iter, 4)
+		queue = model.get_value(iter, 5)
+		immediate = model.get_value(iter, 6)
+		bitratestr = model.get_value(iter, 7)
+		length = model.get_value(iter, 8)
+		directory = model.get_value(iter, 9)
+		#bitrate = model.get_value(iter, 10)
+		fn = model.get_value(iter, 11)
+		country = model.get_value(iter, 12)
+		data[len(data)] = {"user":user, "fn": fn, "position":num, "filename":filename, "directory":directory, "size":size, "speed":speed, "queue":queue, "immediate":immediate, "bitrate":bitratestr, "length":length, "country":country}
+
+			
+	def OnSearchMeta(self, widget):
+		if not self.frame.np.transfers:
+			return
+		data = {}
+		self.ResultsList.get_selection().selected_foreach(self.SelectedResultsAllData, data)
+
+		if data != {}:	
+			self.MetaBox(title=_("Nicotine+: Search Results"), message=_("<b>Metadata</b> for Search Query: <i>%s</i>") % self.text, data=data, modal=True)
+			
 	def OnDownloadFiles(self, widget, prefix = ""):
+		
 		if not self.frame.np.transfers:
 			return
 		for file in self.selected_results:
-			self.frame.np.transfers.getFile(file[0], file[1], prefix)
+			self.frame.np.transfers.getFile(file[0], file[1], prefix, size=file[2], bitrate=file[3], length=file[4])
 	
 	def OnDownloadFilesTo(self, widget):
 		dir = ChooseDir(self.frame.MainWindow, self.frame.np.config.sections["transfers"]["downloaddir"])
@@ -584,7 +831,7 @@ class Search(SearchTab):
 		user, path = self.selected_results[0][:2]
 		path = string.join(path.split("\\")[:-1], "\\") + "\\"
 		self.frame.SetClipboardURL(user, path)
-
+	
 	def OnToggleFilters(self, widget):
 		if widget.get_active():
 			self.Filters.show()
