@@ -5,12 +5,16 @@ import gobject
 import pango
 import time
 import locale
-import os
+import os, sys
 import string
 import re
 import types
 
+from struct import unpack
+import imghdr
+
 from pynicotine import slskmessages
+from pynicotine.utils import _
 
 DECIMALSEP = ""
 
@@ -18,18 +22,32 @@ URL_RE = re.compile("(\\w+\\://[\\w\\.].+?)[\\s\\(\\)]|(www\\.\\w+\\.\\w+.*?)[\\
 PROTOCOL_HANDLERS = {}
 CATCH_URLS = 0
 HUMANIZE_URLS = 0
+USERNAMEHOTSPOTS = 0
+NICOTINE = None
 
-def popupWarning(parent, title, warning):
+def popupWarning(parent, title, warning, icon=None):
 	dlg = gtk.Dialog(title = title, parent = parent,
 		buttons = (gtk.STOCK_OK, gtk.RESPONSE_OK))
 	dlg.set_default_response(gtk.RESPONSE_OK)
-	
+	dlg.set_icon(icon)
 	dlg.set_border_width(10)
 	dlg.vbox.set_spacing(10)
-		
+	hbox = gtk.HBox(spacing=5)
+	hbox.set_border_width(5)
+	hbox.show()
+	dlg.vbox.pack_start(hbox)
+	
+	image = gtk.Image()
+	image.set_padding(0, 0)
+	icon = gtk.STOCK_DIALOG_WARNING
+	image.set_from_stock(icon, 4)
+	image.show()
+	
+	hbox.pack_start(image)	
 	label = gtk.Label()
 	label.set_markup(warning)
-	dlg.vbox.pack_start(label, True, True)
+	label.set_line_wrap(True)
+	hbox.pack_start(label, True, True)
 
 	dlg.vbox.show_all()
 
@@ -51,6 +69,7 @@ def recode2(s):
 	except:
 		return s
 
+		
 def InitialiseColumns(treeview, *args):
 	i = 0
 	cols = []
@@ -58,9 +77,24 @@ def InitialiseColumns(treeview, *args):
 		if c[2] == "text":
 			renderer = gtk.CellRendererText()
 			column = gtk.TreeViewColumn(c[0], renderer, text = i)
+		elif c[2] == "colored":
+			renderer = gtk.CellRendererText()
+			column = gtk.TreeViewColumn(c[0], renderer, text = i, foreground = c[3][0], background = c[3][1])
+		elif c[2] == "edit":
+			renderer = gtk.CellRendererText()
+			renderer.set_property('editable', True)
+			column = gtk.TreeViewColumn(c[0], renderer, text = i)
+		elif c[2] == "combo":
+			renderer = gtk.CellRendererCombo()
+			renderer.set_property('text-column', 0)
+			renderer.set_property('editable', True)
+			column = gtk.TreeViewColumn(c[0], renderer, text = i)
 		elif c[2] == "progress":
 			renderer = gtk.CellRendererProgress()
 			column = gtk.TreeViewColumn(c[0], renderer, value = i)
+		elif c[2] == "toggle":
+			renderer = gtk.CellRendererToggle()
+			column = gtk.TreeViewColumn(c[0], renderer, active = i)
 		else:
 			renderer = gtk.CellRendererPixbuf()
 			column = gtk.TreeViewColumn(c[0], renderer, pixbuf = 0)
@@ -75,14 +109,47 @@ def InitialiseColumns(treeview, *args):
 				column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
 				column.set_fixed_width(c[1])
 			column.set_min_width(0)
-		if len(c) > 3:
+		if len(c) > 3 and type(c[3]) is not list:
 			column.set_cell_data_func(renderer, c[3])
 		column.set_reorderable(True)
+		column.set_widget(gtk.Label(c[0]))
+		column.get_widget().show()
 		treeview.append_column(column)
 		cols.append(column)
 		i += 1
 	return cols
+		
+def PressHeader(widget, event):
+	if event.button != 3:
+		return False
+	columns = widget.get_parent().get_columns()
+	visible_columns = [column for column in columns if column.get_visible()]
+	one_visible_column = len(visible_columns) == 1
+	menu = gtk.Menu()
+	pos = 1
+	for column in columns:
+		title = column.get_title()
+		if title == "":
+			title = _("Column #%i") %pos
+		item = gtk.CheckMenuItem(title)
+		if column in visible_columns:
+			item.set_active(True)
+			if one_visible_column:
+				item.set_sensitive(False)
+		else:
+			item.set_active(False)
+		item.connect('activate', header_toggle, column)
+		menu.append(item)
+		pos += 1
+	menu.show_all()
+	menu.popup(None, None, None, event.button, event.time)
+	return True
 
+def header_toggle(menuitem, column):
+	column.set_visible(not column.get_visible())
+	NICOTINE.SaveColumns()
+
+	
 def ScrollBottom(widget):
 	widget.need_scroll = 1
 	va = widget.get_vadjustment()
@@ -96,10 +163,11 @@ def UrlEvent(tag, widget, event, iter, url):
 		protocol = url[:url.find(":")]
 		if PROTOCOL_HANDLERS.has_key(protocol):
 			if PROTOCOL_HANDLERS[protocol].__class__ is types.MethodType:
-				PROTOCOL_HANDLERS[protocol](url)
+				PROTOCOL_HANDLERS[protocol](url.strip())
 			else:
-				cmd = PROTOCOL_HANDLERS[protocol] % url
-				os.system(cmd)
+				cmd = '%s &' % (PROTOCOL_HANDLERS[protocol] % url)
+
+				os.system( cmd)
 		else:
 			try:
 				import gnome.vfs
@@ -108,12 +176,12 @@ def UrlEvent(tag, widget, event, iter, url):
 				pass
 	tag.last_event_type = event.type
 
-def AppendLine(textview, line, tag = None, timestamp = "%H:%M:%S"):
+def AppendLine(textview, line, tag = None, timestamp = "%H:%M:%S", username=None, usertag=None, scroll=True):
 	def _makeurltag(buffer, tag, url):
 		props = {}
-		#if tag is not None:
-			#color = tag.get_property("foreground_gdk")
-			#props["foreground_gdk"] = color
+		if tag is not None:
+			color = tag.get_property("foreground_gdk")
+			props["foreground_gdk"] = color
 		props["underline"] = pango.UNDERLINE_SINGLE
 		tag = buffer.create_tag(**props)
 		tag.last_event_type = -1
@@ -133,7 +201,9 @@ def AppendLine(textview, line, tag = None, timestamp = "%H:%M:%S"):
 		
 	buffer = textview.get_buffer()
 	linenr = buffer.get_line_count()
-	
+	ME = 0
+	if line.startswith("* "):
+		ME = 1
 	if timestamp:
 		line = "%s %s\n" % (recode(time.strftime(timestamp)), line)
 
@@ -143,18 +213,47 @@ def AppendLine(textview, line, tag = None, timestamp = "%H:%M:%S"):
 		url = match.group()[:-1]
 		urltag = _makeurltag(buffer, tag, url)
 		line = line[match.end()-1:]
-		_append(buffer, start, tag)
+		
+		if USERNAMEHOTSPOTS and username != None and usertag != None and not ME:
+			np = re.compile(re.escape(username))
+			match = np.search(start)
+			if match != None:
+				start2 = start[:match.start()]
+				name = match.group()[:]
+				start = start[match.end():]
+				_append(buffer, start2, tag)
+				_append(buffer, name, usertag)
+				_append(buffer, start, tag)
+			else:
+				_append(buffer, start, tag)
+		else:
+			_append(buffer, start, tag)
 		if url.startswith("slsk://") and HUMANIZE_URLS:
 			url = url.replace("%20", " ")
 		_append(buffer, url, urltag)
 		match = URL_RE.search(line)
 	
+	
 	if line:
-		_append(buffer, line, tag)
+		
+		if USERNAMEHOTSPOTS and username != None and usertag != None and not ME:
+			np = re.compile(re.escape(username))
+			match = np.search(line)
+			if match != None:
+				start = line[:match.start()]
+				name = match.group()[:]
+				line = line[match.end():]
+				_append(buffer, start, tag)
+				_append(buffer, name, usertag)
+				_append(buffer, line, tag)
+			else:
+				_append(buffer, line, tag)
+		else:
+			_append(buffer, line, tag)
 	
 	if not hasattr(scrolledwindow, "need_scroll"):
 		scrolledwindow.need_scroll = 1
-	if bottom and scrolledwindow.need_scroll:
+	if scroll and bottom and scrolledwindow.need_scroll:
 		scrolledwindow.need_scroll = 0
 		gobject.idle_add(ScrollBottom, scrolledwindow)
 	
@@ -186,7 +285,7 @@ class ImageLabel(gtk.HBox):
 			self.button.add(img)
 			self.button.connect("clicked", onclose)
 			self.button.set_relief(gtk.RELIEF_NONE)
-			self.button.set_size_request(16,16)
+			self.button.set_size_request(18,18)
 			self.pack_start(self.button, False, False)
 			self.button.show_all()
 
@@ -232,7 +331,12 @@ class IconNotebook(gtk.Notebook):
 		l2 = ImageLabel(label, self.images["empty"])
 		self.pages.append([page, l, 0, l2])
 		gtk.Notebook.append_page_menu(self, page, l, l2)
-	
+		try:
+			self.set_tab_reorderable(page, True)
+		except:
+			# Old PyGTK2
+			pass
+		
 	def set_image(self, page, status):
 		image = self.images[("empty", "online", "hilite")[status]]
 		for i in self.pages:
@@ -282,24 +386,40 @@ class PopupMenu(gtk.Menu):
 		gtk.Menu.__init__(self)
 		self.frame = frame
 		self.user = None
-		
+		self.useritem = None
 	def setup(self, *items):
 		for item in items:
 			if item[0] == "":
 				menuitem = gtk.MenuItem()
+			elif item[0] == "USER":
+				menuitem = gtk.MenuItem(item[1])
+				self.useritem = menuitem
+				menuitem.set_sensitive(False)
 			elif item[0] == 1:
-				menuitem = gtk.MenuItem()
 				menuitem = gtk.MenuItem(item[1])
 				menuitem.set_submenu(item[2])
 				if item[3] is not None:
 					menuitem.connect("activate", item[3])
+			elif item[0] == "USERMENU":
+				menuitem = gtk.MenuItem(item[1])
+				menuitem.set_submenu(item[2])
+				if item[3] is not None:
+					menuitem.connect("activate", item[3])
+				self.useritem = menuitem
+			elif item[0] == 2:
+				menuitem = gtk.ImageMenuItem(item[1])
+				menuitem.set_submenu(item[2])
+				if item[3] is not None:
+					menuitem.connect("activate", item[3])
+				img = gtk.image_new_from_stock(item[4], gtk.ICON_SIZE_MENU)
+				menuitem.set_image(img)
 			else:
 				if item[0][0] == "$":
 					menuitem = gtk.CheckMenuItem(item[0][1:])
 				elif item[0][0] == "#":
 					menuitem = gtk.ImageMenuItem(item[0][1:])
 					img = gtk.image_new_from_stock(item[2], gtk.ICON_SIZE_MENU)
-        				menuitem.set_image(img)
+					menuitem.set_image(img)
 				elif item[0][0] == "%":
 					menuitem = gtk.ImageMenuItem(item[0][1:])	
 					img = gtk.Image()
@@ -315,7 +435,9 @@ class PopupMenu(gtk.Menu):
 
 	def set_user(self, user):
 		self.user = user
-	
+		if self.useritem:
+			self.useritem.get_child().set_text(user)
+
 	def get_user(self):
 		return self.user
 		
@@ -354,7 +476,7 @@ class PopupMenu(gtk.Menu):
 		self.frame.privatechats.SendMessage(self.user, "\x01VERSION\x01")
 		
 	def OnGivePrivileges(self, widget):
-		text = InputDialog(None, "Give privileges", "Give how many days of privileges do you wish to give this user?" )
+		text = InputDialog(None, _("Give privileges"), _("Give how many days of global privileges to this user?") )
 		if text:
 			try:
 				days = int(text)
@@ -416,12 +538,12 @@ class FastListModel(gtk.GenericTreeModel):
 		return (iter,)
     
 	def on_get_iter(self, path):
-	        '''returns the node corresponding to the given path.  In our
+		'''returns the node corresponding to the given path.  In our
 	        case, the node is the path'''
 		if path[0] < len(self.data):
-		    return path[0]
+			return path[0]
 		else:
-		    return None
+			return None
 
 	def on_get_value(self, iter, column):
 		'''returns the value stored in a particular column for the node'''
@@ -488,12 +610,22 @@ def WriteLog(logfile, logsdir, fn, msg):
 		oldumask = os.umask(0077)
 		if not os.path.exists(logsdir):
 			os.makedirs(logsdir)
-		logfile = open(os.path.join(logsdir, fn.replace(os.sep, "-") + ".log"), 'a', 0)
+		logfile = open(os.path.join(logsdir, fixpath(fn.replace(os.sep, "-")) + ".log"), 'a', 0)
 		os.umask(oldumask)
 	logfile.write("%s %s\n" % (recode(time.strftime("%c")), msg))
 	logfile.flush()
 	return logfile
-
+		
+def fixpath(path):
+	try:
+		if sys.platform == "win32":
+			chars = ["?", "/", "\\", "\"", ":", ">", "<", "|", "*"]
+			for char in chars:
+				path = path.replace(char, "_")
+		return path
+	except:
+		return path
+		
 def Humanize(number):
 	fashion = DECIMALSEP
 	if fashion == "" or fashion == "<None>":
@@ -513,87 +645,87 @@ def Humanize(number):
 	return neg + ret[:-1]
  
 def expand_alias(aliases, cmd):
-    def getpart(line):
-        if line[0] != "(":
-            return ""
-        ix = 1
-        ret = ""
-        level = 0
-        while ix < len(line):
-            if line[ix] == "(":
-                level = level + 1
-            if line[ix] == ")":
-                if level == 0:
-                    return ret
-                else:
-                    level = level - 1
-            ret = ret + line[ix]
-            ix = ix + 1
-        return ""
+	def getpart(line):
+		if line[0] != "(":
+			return ""
+		ix = 1
+		ret = ""
+		level = 0
+		while ix < len(line):
+			if line[ix] == "(":
+				level = level + 1
+			if line[ix] == ")":
+				if level == 0:
+					return ret
+				else:
+					level = level - 1
+			ret = ret + line[ix]
+			ix = ix + 1
+		return ""
 
-    if not cmd:
-        return None
-    if cmd[0] != "/":
-        return None
-    cmd = cmd[1:].split(" ")
-    if not aliases.has_key(cmd[0]):
-        return None
-    alias = aliases[cmd[0]]
-    ret = ""
-    i = 0
-    while i < len(alias):
-        if alias[i:i+2] == "$(":
-            arg=getpart(alias[i+1:])
-            if not arg:
-                ret = ret + "$"
-                i = i + 1
-                continue
-            i = i + len(arg) + 3
-            args = arg.split("=",1)
-            if len(args) > 1:
-                default = args[1]
-            else:
-                default = ""
-            args = args[0].split(":")
-            if len(args) == 1:
-                first = last = int(args[0])
-            else:
-               if args[0]:
-                   first = int(args[0])
-               else:
-                   first = 1
-               if args[1]:
-                   last = int(args[1])
-               else:
-                   last = len(cmd)
-            v = string.join(cmd[first:last+1])
-            if not v: v = default
-            ret = ret + v
-        elif alias[i:i+2] == "|(":
-            arg = getpart(alias[i+1:])
-            if not arg:
-                ret = ret + "|"
-                i = i + 1
-                continue
-            i = i + len(arg) + 3
-            for j in range(len(cmd)-1, -1, -1):
-                arg = arg.replace("$%i" % j, cmd[j])
-            arg = arg.replace("$@", string.join(cmd[1:], " "))
-            stdin, stdout = os.popen2(arg)
-            v = stdout.read().split("\n")
-            r = ""
-            for l in v:
-                l = l.strip()
-                if l:
-                    r = r + l + "\n"
-            ret = ret + r.strip()
-            stdin.close()
-            stdout.close()
-            os.wait()
-        else:
-            ret = ret + alias[i]
-            i = i + 1
-    return ret
+	if not cmd:
+		return None
+	if cmd[0] != "/":
+		return None
+	cmd = cmd[1:].split(" ")
+	if not aliases.has_key(cmd[0]):
+		return None
+	alias = aliases[cmd[0]]
+	ret = ""
+	i = 0
+	while i < len(alias):
+		if alias[i:i+2] == "$(":
+			arg=getpart(alias[i+1:])
+			if not arg:
+				ret = ret + "$"
+				i = i + 1
+				continue
+			i = i + len(arg) + 3
+			args = arg.split("=",1)
+			if len(args) > 1:
+				default = args[1]
+			else:
+				default = ""
+			args = args[0].split(":")
+			if len(args) == 1:
+				first = last = int(args[0])
+			else:
+				if args[0]:
+					first = int(args[0])
+				else:
+					first = 1
+				if args[1]:
+					last = int(args[1])
+				else:
+					last = len(cmd)
+			v = string.join(cmd[first:last+1])
+			if not v: v = default
+			ret = ret + v
+		elif alias[i:i+2] == "|(":
+			arg = getpart(alias[i+1:])
+			if not arg:
+				ret = ret + "|"
+				i = i + 1
+				continue
+			i = i + len(arg) + 3
+			for j in range(len(cmd)-1, -1, -1):
+				arg = arg.replace("$%i" % j, cmd[j])
+			arg = arg.replace("$@", string.join(cmd[1:], " "))
+			stdin, stdout = os.popen2(arg)
+			v = stdout.read().split("\n")
+			r = ""
+			for l in v:
+				l = l.strip()
+				if l:
+					r = r + l + "\n"
+			ret = ret + r.strip()
+			stdin.close()
+			stdout.close()
+			os.wait()
+		else:
+			ret = ret + alias[i]
+			i = i + 1
+	return ret
 
 def EncodingsMenu(np, section = None, entry = None):
 	if section and entry and np.config.sections["server"][section].has_key(entry):
@@ -613,3 +745,248 @@ def SaveEncoding(np, section, entry, encoding):
 	elif np.config.sections["server"][section].has_key(entry):
 		del np.config.sections["server"][section][entry]
 	np.config.writeConfig()
+	
+class ImportWinSlskConfig:
+	def __init__(self, config, Path, Queue, Login, Rooms, BuddyList, BanList, IgnoreList, UserInfo, UserImage):
+		self.config = config
+		self.Path = Path
+		self.Queue = Queue
+		self.Login = Login
+		self.Rooms = Rooms
+		self.BuddyList = BuddyList
+		self.BanList = BanList
+		self.IgnoreList = IgnoreList
+		self.UserInfo = UserInfo
+		self.UserImage = UserImage
+		
+	def Run(self):
+		if not self.check_slskdir():
+			return 0
+		if not self.Queue and not self.Login and not self.Rooms and not self.BuddyList and not self.IgnoreList and not self.BanList and not self.UserInfo and not self.UserImage:
+			return 2
+
+		if self.Queue:
+			# Get download queue
+			windownloads = self.get_downloads(self.winpath('queue2.cfg'))
+		
+			for i in windownloads:
+				if not i in self.config.sections["transfers"]["downloads"]:
+					#print i
+					self.config.sections["transfers"]["downloads"].append(i)
+			
+		if self.BuddyList:
+			# Getting userlist
+			users = self.get_basic_config(self.winpath('hotlist.cfg'))
+		
+			for i in users:
+				if not self.is_in_user_list(i,self.config.sections["server"]["userlist"]):
+					#print [i,'']
+					self.config.sections["server"]["userlist"].append([i,''])
+		
+		if self.Login:
+			# Get login and password
+			(login,passw) = self.get_user_and_pass(self.winpath('login.cfg'))
+			self.config.sections["server"]["login"] = login
+			self.config.sections["server"]["passw"] = passw
+		
+		if self.Rooms:
+			# Get the list of autojoined chatrooms
+			chatrooms = self.get_basic_config(self.winpath('chatrooms.cfg'))
+			chatrooms.append('nicotine')
+		
+			for i in chatrooms:
+				if i not in self.config.sections["server"]["autojoin"]:
+					self.config.sections["server"]["autojoin"].append(i)
+		
+		if self.BanList:
+			# Get the list of banned users
+			banlist = self.get_basic_config(self.winpath('dlbans.cfg'))
+		
+			for i in banlist:
+				if i not in self.config.sections["server"]["banlist"]:
+					self.config.sections["server"]["banlist"].append(i)
+		
+		if self.IgnoreList:
+			# Get the list of ignored users
+			ignorelist = self.get_basic_config(self.winpath('ignores.cfg'))
+		
+			for i in ignorelist:
+				if i not in self.config.sections["server"]["ignorelist"]:
+					self.config.sections["server"]["ignorelist"].append(i)
+		if self.UserInfo:
+			# Get userinfo and image
+			(descr, imgdata) = self.get_userinfo(self.winpath('userinfo.cfg'))
+			if descr:
+				self.config.sections["userinfo"]['descr'] = descr.__repr__()
+		if self.UserImage:
+			(descr, imgdata) = self.get_userinfo(self.winpath('userinfo.cfg'))
+			if imgdata:
+				img = self.save_image(imgdata)
+				if img != "":
+					self.config.sections["userinfo"]['pic'] = img
+		return 1
+  
+	def check_slskdir(self):
+		# we check if the file queue2.cfg exists under the slsk dir
+		queue2path = os.path.join(self.Path, 'queue2.cfg')
+		
+		if not os.access(queue2path, os.F_OK):
+			return 0
+		return 1
+
+
+	def winpath(self, file):
+		return os.path.join(self.Path, file)
+
+
+	def get_downloads(self, fname):
+		"""Returns the list of downloads in the queue2.cnf file
+		
+		The windows slsk queue2.cnf file format:
+		
+		- little-endian, wordsize=4
+		- The file starts with 3 ints (4bytes per int).
+		- First 2 ints are unknown
+		- The 3rd one is the number of entries in the file
+		
+		Then for each download entry:
+		
+		- The length of the username, followed by the username
+		- The length of the remote filename, followed by the remote filename
+		- The length of the local dir, followed by the local dir
+		
+		The file can contain some unknown stuff at the end."""
+		
+		infile = open(fname, 'r')
+		intsize = 4
+		downloads = []
+		
+		str = infile.read(3*intsize)
+		
+		i1, i2, n_entries = unpack("iii", str)
+		
+		for i in range(n_entries):
+			length = unpack("i", infile.read(intsize))[0]
+			uname = infile.read(length)
+			length = unpack("i", infile.read(intsize))[0]
+			remfile = infile.read(length)
+			length = unpack("i", infile.read(intsize))[0]
+			localdir = infile.read(length)
+			downloads.append([uname, remfile, localdir])
+			
+		infile.close()
+		return downloads
+    
+
+	def get_user_and_pass(self, fname):
+		"""Returns a (user,passwd) tuple. 
+		
+		File format of login.cfg (see also get_downloads):
+		
+		- two unknown ints at the beginning
+		- the number of bytes of the username, followed by the username
+		- the number of bytes from the passwd, followed by the passwd
+		- followed by some undetermined bytes"""
+		
+		
+		infile = open(fname, 'r')
+		intsize = 4
+		downloads = []
+		
+		str = infile.read(3*intsize)
+		
+		i1, i2, length = unpack("iii", str)
+		
+		user = infile.read(length)
+		length = unpack("i", infile.read(intsize))[0]
+		passwd = infile.read(length)
+			
+		infile.close()
+		return (user, passwd)
+
+
+	def get_basic_config(self, fname):
+		"""Works for userlist, chatrooms, dlbans, etc.
+		
+		
+		The hotlist.cnf file format (see get_downloads for more info):
+		
+		- The file starts with an int: the number of users in the list
+		
+		Then for each user:
+		
+		- The length of the username, followed by the username"""
+		
+		infile = open(fname, 'r')
+		intsize = 4
+		users = []
+		
+		str = infile.read(1*intsize)
+		
+		n_entries = unpack("i", str)[0]
+		for i in range(n_entries):
+			length = unpack("i", infile.read(intsize))[0]
+			user = infile.read(length)
+			users.append(user)
+			
+		infile.close()
+		return users
+
+
+	def get_userinfo(self, fname):
+		"""The userinfo file format:
+		
+		- an int with the size of the text part, followed by the text part
+		- one byte, an indication for an image. 0 is no image, 1 is image
+		- an int with the image size, followed by the image data """
+		
+		infile = open(fname, 'r')
+		intsize = 4
+		
+		imgdata = None
+		
+		str = infile.read(intsize)
+		length = unpack("i", str)[0]
+		descr = infile.read(length)
+		descr = string.replace(descr, "\r", "")
+		has_image = unpack("b",infile.read(1))[0]
+		if has_image:
+			length = unpack("i", infile.read(intsize))[0]
+			imgdata = infile.read(length)
+			
+		infile.close()
+		return (descr, imgdata)
+
+
+	def save_image(self, imgdata):
+		"""Save IMGDATA to file and return the filename or None."""
+		if not imgdata:
+			print "No image data.  No image saved."
+			return ""
+		
+		extension = imghdr.what(None, imgdata)
+		if not extension:
+			# how to print to stderr in python?  (no real problem here)
+			print "Could not determine image type.  Image not saved."
+			return ""
+		
+		fname = os.path.abspath(self.config.filename + '-userinfo.' + extension)
+		outfile = open(fname, "w")
+		outfile.write(imgdata)
+		outfile.close()
+		print "Wrote image to \"%s\"" % (fname)
+		return fname
+
+
+	def is_in_user_list(self, user, user_list):
+		"""Checks if USER is in USER_LIST, ignoring the comment field"""
+		for i in user_list:
+			if type(i) == type(''):
+				sys.stderr.write("\nError: The nicotine userlist is in an old pyslsk format.\n" +
+				"Please run Nicotine once before running this script.\n" +
+				"Config file not updated.\n")
+				break
+			if user == i[0]:
+				return 1
+		return 0
+
